;;----------------------------------------LICENSE NOTICE----------------------------------------
;;  Code & Gfx Copyright (C) 2019 ByteCode Studios (@ByteCodeStudios)
;;  Music (C) 2019 McKlain (@elmcklain)
;;  This file is part of Jackstrad: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;----------------------------------------------------------------------------------------------

.include "man/man_memory/entity.h.s"
.include "man/man_memory/pointer_array.h.s"
.include "cpctelera.h.s"

DefineEntity enemy,10,25,8,16,_enemy_sp_1,0,0, e_ai_st_stand_by, 2,6,enemy_collision, 1, _enemy_sp_4
DefineEntity plant,10,25,15,28,_plant_sp,0,0, e_ai_st_stand_by, 2,6,plant_collision, 1, _enemy_sp_4

;;CREA UN NUEVO ENEMIGO EN LA POSICION INDICADA
;;ENTRADA -> bc(x,y)
;;e->prevSt
;;d->direccion
enemy_create::
	push bc
	push de
	push hl

	ld hl, #enemy
	call entityMan_create
	pop af
	pop de
	pop bc

	;Posicion del enemigo
	ld e_x(ix), b
	ld e_y(ix), c
	call player_setPosition

	ld e_ai_prev_st(ix), e
	ld e_direction(ix), d
	push hl
	push af
	pop hl
	ld e_ai_patrol_step_l(ix), l
	ld e_ai_patrol_step_h(ix), h

	pop hl
	ld e_life(ix), #3
	
	push ix
	push hl
	call enemyStand_setAnimation
	pop hl
	pop ix

	push ix
	call man_render_add
	call man_physics_add
    call man_ia_add
	pop ix
	ret

enemy_remove2::
	call manPointer_removeRender
	call manPointer_removePhysics
	call manPointer_removeIA
	ret

;;ELIMINA UN ENEMIGO
;;ENTRADA -> IX(Enemigo a borrar)
enemy_remove::
    push ix

    ;Borramos el render
    call manPointer_removeRender
	ld (render_pointer_aux), hl

	;Borramos las fisicas
	call manPointer_removePhysics
	ld (physics_pointer_aux), hl

    ;Borramos las fisicas
	call manPointer_removeIA
	ld (ia_pointer_aux), hl

	;Borramos la entidad
	call entityMan_remove
	pop ix

    ;Comprobamos si es null
	ld a, e_w(ix)
	or a
	ret z

	;Actualizo las fisicas en la entidad
	ld a, e_physics_h(ix)
	or a
	jr z, no_physics_update

	ld hl,(physics_pointer_aux)
	call manPointer_updatePhysics

;No tiene fisica
no_physics_update:
	ld a, e_render_h(ix)
	or a
	jr z, no_render_update

	ld hl,(render_pointer_aux)
	call manPointer_updateRender

;No tiene render
no_render_update:
	ld a, e_ia_h(ix)
	or a
	ret z

	ld h, e_ia_h(ix)
	ld l, e_ia_l(ix)
	call manPointer_updateIA
ret

;;FUNCION QUE MANEJA COMO DEBE ACTUAR 
;;EL ENEMIGO CUANDO COLISIONA
enemy_collision::
	;Compruebo si es un proyectil
    call type_checkProjectile
    or a
    ret nz

	;Decrementamos la vida del personaje
	ld a, e_life(ix)
	dec a
	ld e_life(ix), a
	or a
	jr nz, no_dead_enemy
	
	;Borramos el sprite de la pantalla
	call render_erase_sprite
	call render_switchBuffer
	call render_erase_current_sprite
	call render_switchBuffer
    call enemy_remove

	;Interrupciones
	call time_addInterruption2
    ret

no_dead_enemy:
	ld a, #6
	ld e_draw(ix), a
	ret


plant_create::
	push af
	push bc
	push de
	ld hl, #plant
	call entityMan_create
	pop de
	pop bc
	pop af

	;Posicion del corazon
	ld e_x(ix), b
	ld e_y(ix), c
	call player_setPosition

	push ix
	call man_render_add
	call man_physics_add
	pop ix
	ret

plant_collision::
	;Comprobamos que es el jugador
	call type_checkPlayer
    or a
    ret nz

	call entityMan_getArray
	;Borramos el sprite
	call render_erase_sprite
	call render_switchBuffer
	call render_erase_current_sprite
	call render_switchBuffer

	call plantScreen
	ld a, #700
wait_press2:
		push af
		call cpct_waitVSYNC_asm
		call cpct_waitVSYNC_asm
		call cpct_waitVSYNC_asm
		halt
		pop af

		dec a
		jr nz, wait_press2

	ld a, (num_dead)
	inc a
	ld (num_dead), a

	call gameOver
	ld a, #950

wait_press:
	push af
	call cpct_waitVSYNC_asm
	call cpct_waitVSYNC_asm
	call cpct_waitVSYNC_asm
	halt
	pop af

	dec a
	jr nz, wait_press
	
	;Reiniciamos el juego
	ld a, #0
	ld (_floor), a

	ld a, #0
	ld (_keys),a

	call game_clearData 
	call man_map_loadNextFloor
	ret