;;----------------------------------------LICENSE NOTICE----------------------------------------
;;  Code & Gfx Copyright (C) 2019 ByteCode Studios (@ByteCodeStudios)
;;  Music (C) 2019 McKlain (@elmcklain)
;;  This file is part of Jackstrad: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;----------------------------------------------------------------------------------------------

.include "main.h.s"
.include "man/man_memory/entity.h.s"
.include "cpctelera.h.s"
.include "sys/ai_control.h.s"
.include "man/patrol.h.s"

contador: .db 06

int_handler::
   push hl
   push af
   push de
   push bc
   push ix

   ld a, (contador)
   dec a
   jr nz, _cont

_zero:
   call cpct_akp_musicPlay_asm
   call cpct_scanKeyboard_if_asm
   ld a, #6

_cont:
   ld (contador), a
   pop ix
   pop bc
   pop de
   pop af
   pop hl

   ei
   reti

set_int_handler:
   call cpct_waitVSYNC_asm
   halt
   halt
   call cpct_waitVSYNC_asm

   ld hl, #0x38
   ld (hl), #0xC3
   inc hl
   ld  (hl), #<int_handler
   inc hl
   ld (hl), #>int_handler
   inc hl
   ld (hl), #0xC9

   ret


;;Limpia los array de los diferentes sistemas
game_clearData::
      call entityMan_init
      call man_render_init
      call man_ia_init
      call man_patrol_init
      call man_physics_init
      call man_col_init
      call man_projectile_init
      ret

game_init::
      ld de, #_song
      call cpct_akp_musicInit_asm
      ;;call cpct_disableFirmware_asm
      call set_int_handler
      ld a, #0
      ld a, (num_dead)
      call render_init
      ;Cargamos el menú
      jp _menu
    
_continue:
      ;Inicializamos sistemas
      call man_map_init
      call render_init
      call sys_ai_control_init
      call game_clearData 

      call man_map_loadNextFloor
      ret
_menu:
   call getMenu

_continuemenu:
   ;;call cpct_scanKeyboard_asm
   ld hl, #Key_1 ;;Comprobamos si la tecla 1 ha sido pulsada
   call cpct_isKeyPressed_asm
   jr z, no_key1
   jr nz, _continue ;;Si se ha pulsado, empieza el juego

no_key1:
   ld hl, #Key_2
   call cpct_isKeyPressed_asm
   jr z, no_key2
   ;;Clean the screen
   jr nz, controls

no_key2:
   ld hl, #Key_3
   call cpct_isKeyPressed_asm
   jr z, _continuemenu
   ;;Clean the screen
   jr nz, credits
   
controls:
   call getControl
controls_loop:
   call cpct_isAnyKeyPressed_asm
   jr z, controls_loop
   jr  _menu

credits:
   call getCredits
_credits2:
   call cpct_isAnyKeyPressed_asm
   jr z, _credits2
   jr nz, _menu


game_update::
   ;;Recibimos el movimiento del personaje
   call sys_control_update
   call man_col_call

   ;;Control de la ia
   call sys_ai_control_update
 
   ;;Actualizamos las fisicas
   call physics_update

   ;;Comprobamos las colisiones
   call collision_update
   
   ;;Renderizamos la escena
   call render_update
   call render_switchBuffer
   call time_update

   jr   game_update




