;;----------------------------------------LICENSE NOTICE----------------------------------------
;;  Code & Gfx Copyright (C) 2019 ByteCode Studios (@ByteCodeStudios)
;;  Music (C) 2019 McKlain (@elmcklain)
;;  This file is part of Jackstrad: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;----------------------------------------------------------------------------------------------

.include "man/man_memory/entity.h.s"
.include "man/man_memory/pointer_array.h.s"
.include "cpctelera.h.s"

DefineEntity hearth,0,0,6,12,_hearth_sp_0,0,0, e_ai_st_stand_by, 0,0,hearth_collision, 6,_hearth_sp_1

;;CREA UN NUEVO CORAZON EN LA POSICION INDICADA
;;ENTRADA -> bc(x,y)
;;			 a(Habitacion donde esta la llave)
hearth_create::
	push af
	push bc
	push de
	ld hl, #hearth
	call entityMan_create
	pop de
	pop bc
	pop af

	ld e_hearthRoom(ix), a

	;Posicion del corazon
	ld e_x(ix), b
	ld e_y(ix), c
	call player_setPosition

	push ix
	call man_render_add
	call man_physics_add
	pop ix
	ret

;;ELIMINA AL JUGADOR
;;ENTRADA -> IX(Enemigo a borrar)
hearth_remove::
	push ix

    ;Borramos el render
    call manPointer_removeRender
	ld (render_pointer_aux), hl

	;Borramos las fisicas
	call manPointer_removePhysics
	ld (physics_pointer_aux), hl

	;Borramos la entidad
	call entityMan_remove
	pop ix

    ;Comprobamos si es null
	ld a, e_w(ix)
	or a
	ret z

	;Actualizo las fisicas en la entidad
	ld hl, (physics_pointer_aux)
	or h
	jr z, no_physics_update
	call manPointer_updatePhysics

;No tiene fisica
no_physics_update:
	ld hl, (render_pointer_aux)
	or h
	jr z, no_render_update
	call manPointer_updateRender

;No tiene render
no_render_update:
	ld h, e_ia_h(ix)
	or h
	ret z
	ld l, e_ia_l(ix)
	call manPointer_updateIA
    ret

hearth_collision::
    ;Comprobamos que es el jugador
	call type_checkPlayer
    or a
    ret nz

    call player_incLife
    call updateHearthRoom

    ;Borramos el sprite de la pantalla
	call render_erase_sprite
	call render_switchBuffer
	call render_erase_current_sprite
	call render_switchBuffer
	call hearth_remove

	call interface_draw
	call time_addInterruption
    ret

;;Comprobamos si en que habitacion tenemos 
;;que borrar la llave
updateHearthRoom::
	ld a, e_hearthRoom(ix)

	cp #0
	jr z, deleteHearthRoom0

	cp #1
	jr z, deleteHearthRoom1
	
	cp #2
	jr z, deleteHearthRoom2

	cp #3
	jr z, deleteHearthRoom3
	
	cp #4
	jr z, deleteHearthRoom4

	cp #5
	jr z, deleteHearthRoom5
	ret

deleteHearthRoom0:
	ld a, #0
	ld (_hearth_room0), a
	ret

deleteHearthRoom1:
	ld a, #0
	ld (_hearth_room1), a
	ret

deleteHearthRoom2:
	ld a, #0
	ld (_hearth_room2), a
	ret

deleteHearthRoom3:
	ld a, #0
	ld (_hearth_room3), a
	ret

deleteHearthRoom4:
	ld a, #0
	ld (_hearth_room4), a
	ret

deleteHearthRoom5:
	ld a, #0
	ld (_hearth_room5), a
	ret
