;;----------------------------------------LICENSE NOTICE----------------------------------------
;;  Code & Gfx Copyright (C) 2019 ByteCode Studios (@ByteCodeStudios)
;;  Music (C) 2019 McKlain (@elmcklain)
;;  This file is part of Jackstrad: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;----------------------------------------------------------------------------------------------

.include "man/man_memory/entity.h.s"
.include "man/man_memory/pointer_array.h.s"
.include "cpctelera.h.s"

DefineEntity key,5,20,4,16,_key_sp_0,0,0, e_ai_st_stand_by, 2,6,key_collision, 6,_key_sp_1

_keys:: .db 0

;;CREA UNA NUEVA LLAVE EN LA POSICION INDICADA
;;ENTRADA -> bc(x,y)
;;			 a(Habitacion donde esta la llave)
key_create::
	push af
	push bc
	push de
	ld hl, #key
	call entityMan_create
	pop de
	pop bc
	pop af

	ld e_keyRoom(ix), a

	;Posicion de la llave
	ld e_x(ix), b
	ld e_y(ix), c
	call player_setPosition

	push ix
	call man_render_add
	call man_physics_add
	pop ix
	ret

;;ELIMINA AL JUGADOR
;;ENTRADA -> IX(Enemigo a borrar)
key_remove::
	push ix

    ;Borramos el render
    call manPointer_removeRender
	ld (render_pointer_aux), hl

	;Borramos las fisicas
	call manPointer_removePhysics
	ld (physics_pointer_aux), hl

	;Borramos la entidad
	call entityMan_remove
	pop ix

    ;Comprobamos si es null
	ld a, e_w(ix)
	or a
	ret z

	ld a, e_physics_h(ix)
	or a
	jr z, no_physics_update

	;Actualizo las fisicas en la entidad
	ld hl, (physics_pointer_aux)
	jr z, no_physics_update
	call manPointer_updatePhysics

;No tiene fisica
no_physics_update:
	ld a, e_render_h(ix)
	or a
	jr z, no_render_update

	ld hl, (render_pointer_aux)
	jr z, no_render_update
	call manPointer_updateRender

;No tiene render
no_render_update:
	ld a, e_ia_h(ix)
	or a
	ret z
	
	ld h, e_ia_h(ix)
	ld l, e_ia_l(ix)
	call manPointer_updateIA
ret

key_collision::
	;Comprobamos que es el jugador
	call type_checkPlayer
    or a
    ret nz

	;Incrementamos el numero de llaves
	ld a, (_keys)
	inc a
	ld (_keys), a

	;Comprobamos si tenemos que cambiar de nivel
	cp a, #4
	jr z, nextFloor
	
	call updateKeyRoom

	;Borramos el sprite de la pantalla
	call render_erase_sprite
	call render_switchBuffer
	call render_erase_current_sprite
	call render_switchBuffer
	call key_remove
	call interface_draw
	call time_addInterruption
    ret

nextFloor:
	ld a, #0
	ld (_keys), a

	ld a,(_floor)
	inc a
	ld (_floor), a
	call man_map_loadNextFloor
	ret


;;Comprobamos si en que habitacion tenemos 
;;que borrar la llave
updateKeyRoom::
	ld a, e_keyRoom(ix)

	cp #0
	jr z, deleteKeyRoom0

	cp #1
	jr z, deleteKeyRoom1
	
	cp #2
	jr z, deleteKeyRoom2

	cp #3
	jr z, deleteKeyRoom3
	
	cp #4
	jr z, deleteKeyRoom4

	cp #5
	jr z, deleteKeyRoom5
	ret

deleteKeyRoom0:
	ld a, #0
	ld (_key_room0), a
	ret

deleteKeyRoom1:
	ld a, #0
	ld (_key_room1), a
	ret

deleteKeyRoom2:
	ld a, #0
	ld (_key_room2), a
	ret

deleteKeyRoom3:
	ld a, #0
	ld (_key_room3), a
	ret

deleteKeyRoom4:
	ld a, #0
	ld (_key_room4), a
	ret

deleteKeyRoom5:
	ld a, #0
	ld (_key_room5), a
	ret
