;;----------------------------------------LICENSE NOTICE----------------------------------------
;;  Code & Gfx Copyright (C) 2019 ByteCode Studios (@ByteCodeStudios)
;;  Music (C) 2019 McKlain (@elmcklain)
;;  This file is part of Jackstrad: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;----------------------------------------------------------------------------------------------
.include "entity.h.s"
.include "cpctelera.h.s"

;;Numero maximo de entidades
max_entities == 50

;;Numero de entidades actuales
_num_entities:: .db 0

;;Array de entidades
DefineArrayEntity _entity_array, max_entities

;;Siguiente elemento a escribir
_next_element:: .dw _entity_array 

;;Ultimo element del array
_last_element:: .dw _entity_array

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Obtiene el array de entidades
;;REGISTROS DESTRUIDOS: ix
;;SALIDA:
;;      ix -> Puntero a la primera posicion del array
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entityMan_getArray::
    ld ix, #_entity_array
    ret

entityMan_cleanArray::
    ld de, #_entity_array
    ld a, #0
    ld bc, #max_entities
    call cpct_memset_asm
    ret 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Obtiene el array de entidades
;;REGISTROS DESTRUIDOS: ix
;;ENTRADA: a -> n(numero de entidad)
;;SALIDA:
;;      ix -> Puntero a la primera posicion del array
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entityMan_getEntityN::
    ld ix, #_entity_array

_getloop:
    ld bc,#sizeof_e
    add ix,bc
    dec a
    jr nz,_getloop
    ret

;;Obtiene en hl la ultima entidad
entityMan_getLast::
  ld hl, #_entity_array
  ld a, (_num_entities)

_last_loop:
    dec a
    ret z
    
    ld bc,#sizeof_e
    add hl, bc
  
    jr nz,_last_loop
    ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Obtiene el numero de entidades
;;REGISTROS DESTRUIDOS: a
;;SALIDA:
;;      a -> Numero entidades
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
entityMan_getNum::
    ld a, (_num_entities)
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Obtiene la ultima entidad
;;REGISTROS DESTRUIDOS: hl
;;SALIDA:
;;      hl -> Numero entidades
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
entityMan_getLast_pointer::
    ld de, #_last_element
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Entity Manager Init
;; Inicializa el manager de entidades, pone el array a 
;; 0 entidades y listo para empezar a crear nuevas
;; Entrada: -
;; Destruye: AF, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entityMan_init::
    xor a
    ld (_num_entities), a

    ld hl, #_entity_array
    ld (_next_element), hl

    call entityMan_cleanArray
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Añade una nueva entidad al array sin inicializar
;;
;;Destruye: HL, DE, BC, 
;;Devuelve: DE = Puntero al nuevo elemento
;;          BC = Tamaño nueva entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entityMan_new::
    ld hl, #_num_entities
    inc (hl)

    ld hl, (_next_element)
    ld (_last_element), hl
    ld d, h
    ld e, l
    ld bc, #sizeof_e
    add hl, bc
    ld (_next_element), hl

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;REGISTRA EN MEMORIA UNA NUEVA ENTIDAD
;;REGISTROS DESTRUIDOS: hl,de,bc
;;ENTRADA:
;;      hl -> Puntero que contiene la nueva entidad
;;SALIDA 
;;      ix -> Puntero a la entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entityMan_create::
    ld de, (_next_element)
    push de

    ;;Guardamos hl porque se pierde
    push hl          
    call entityMan_new
    
    ;; ix = de(puntero a la nueva entidad)
    ld__ixh_d
    ld__ixl_e

    pop hl
    ldir

    ;Devuelve en hl el puntero
    pop de
    ex de,hl
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;DESTRUE DE MEMORIA UNA ENTIDAD
;;REGISTROS DESTRUIDOS: hl,de,bc
;;ENTRADA:
;;      ix -> Puntero que contiene la entidad a borrar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
entityMan_remove::
    push ix
    pop de

    ;Intercambiamos los valores
    ld hl, (_last_element)
    ld bc, #sizeof_e
    ldir

    ;Ponemos a 0 el contenido del ultimo elemento
    ld de, (_last_element)
    ld a, #0
    ld bc, #sizeof_e
    call cpct_memset_asm

    ;El siguiente elemento apunta al ultimo
    ld hl, (_last_element)
    ld (_next_element), hl

    ;Decrementamos el numero de entidades
    ld hl, #_num_entities
    dec (hl)

    call entityMan_getLast
    ld (_last_element), hl
    ret
    


