;;----------------------------------------LICENSE NOTICE----------------------------------------
;;  Code & Gfx Copyright (C) 2019 ByteCode Studios (@ByteCodeStudios)
;;  Music (C) 2019 McKlain (@elmcklain)
;;  This file is part of Jackstrad: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;----------------------------------------------------------------------------------------------

.include "pointer_array.h.s"
.include "cpctelera.h.s"
.include "entity.h.s"

_num_col_entities:: .db 0 

_max_col == 150

sizeof_collision = 5 + sizeof_pointer
pointer_col = sizeof_pointer
pointer_x = sizeof_pointer + 1
pointer_y = sizeof_pointer + 2
pointer_w = sizeof_pointer + 3
pointer_hh = sizeof_pointer + 4

;;Array de entidades
DefinePointerArray _col_array, _max_col

_next_element_col:: .dw _col_array

_type_element:: .db 0
_x_element:: .db 0
_y_element:: .db 0
_w_element:: .db 0
_h_element:: .db 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Obtiene el numero de entidades
;;REGISTROS DESTRUIDOS: a
;;SALIDA:
;;      a -> Numero entidades
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
man_col_getNum::
    ld a, (_num_col_entities)
    ret

;;Obtiene el array con las entidades con colision
;;ENTRADA: hl -> puntero a entidad con colision
man_col_getArray::
    ld ix, #_col_array
    ret

;;Limpia todo el array
man_col_clean::
    ld de, #_col_array
    ld a, #0
    ld bc, #_max_col
    call cpct_memset_asm
ret

man_col_clean_pos::
    ld a, #0
    ld bc, #sizeof_collision
    call cpct_memset_asm
ret


;;Inicializa el array de colision
man_col_init::
    ;Numero de entidades a 0
    xor a
    ld (_num_col_entities), a

    ld hl, #_col_array
    ld (_next_element_col), hl

    call man_col_clean
    ret

;;Añade una nueva entidad al array de colision
;;ENTRADA: hl -> puntero a entidad con colision
;;          a -> Tipo de entidad con la que ha colisionado
;;         bc -> (x,y) de la otra entidad
;;         de -> (w,h) de la otra entidad
;;SALIDA: hl -> puntero a entidad con colision
man_col_add::
    push hl
    call man_col_getArray
    
    ;;Guardo la entidad en memoria
    ld  ix, (_next_element_col)
    ld  pointer_l(ix), l
    ld  pointer_h(ix), h 
    ld  pointer_col(ix), a
    ld  pointer_x(ix), b
    ld  pointer_y(ix), c
    ld  pointer_w(ix), d
    ld  pointer_hh(ix), e

    ;;Incremento el numero de entidades
    ld hl, #_num_col_entities
    inc (hl)
  
    ;Calcula la siguiente posicion
    ld hl,  (_next_element_col)
    ld bc,  #sizeof_collision
    add hl, bc
    ld (_next_element_col), hl

    pop hl
    ret

man_col_call::
    call man_col_getNum
    ;;Compruebo si hay entidades
    or a
	ret z
_col_loop:
    ;;Guardamos los valores de
    ;;los registros
    push af
    call man_col_getLast
    ld a, e_w(ix)
    or a

    jr z, _ret_col

    ;Cargo la colision
    ld h,e_colision_h(ix)
    ld l,e_colision_l(ix)
    
    ;Llamo a la funcion
    ld bc, #_ret_col
    push bc
    ;Cargamos la informacion que le pasamos
    ld a, (_x_element)
    ld b, a
    ld a, (_y_element)
    ld c, a
    ld a, (_w_element)
    ld d, a
    ld a, (_h_element)
    ld e, a
    ld a, (_type_element)
    jp(hl)

_ret_col:
    pop af
    ld b, a
     
    ;;Compruebo si hay entidades
    call man_col_getNum
    or a
	ret z
    
    ld a, b
    dec a

    jr nz,_col_loop
    call man_col_init
    ret

;;Obtiene la ultima direccion de memoria
;;SALIDA -> hl (Ultima direccion de memoria)
man_col_getLast::
    ld a, #0
    ld (_type_element), a
    call man_col_getNum
    ld hl, #0

    ;;Compruebo si hay entidades
    or a
	ret z
    ld ix, #_col_array
getLastLoop:
    push af
    ld a, pointer_h(ix)
    ld b, pointer_l(ix)

    ;Comparo el bit alto
    cp h
    jr nc, compareLow

;Comprobamos si volvemos al loop o devolvemos los valores
ret_loop:
    ld bc, #sizeof_collision
    add ix, bc
    pop af
    dec a
    jr nz, getLastLoop

    push de

    push hl
    pop de
    ld__ixh_d
    ld__ixl_e

    pop de 
    call man_col_clean_pos
    ld a,(_type_element)
    ret

   ;Comparo el bit bajo
compareLow:
    ld a,b
    cp l
    jr nc, setHL
    jr ret_loop

;Cambiamos el valor hl
setHL:
    ld h, pointer_h(ix)
    ld l, pointer_l(ix)
    ;TIPO
    ld a, pointer_col(ix)
    ld (_type_element), a

    ;POSICION
    ld a, pointer_x(ix)
    ld (_x_element), a
    ld a, pointer_y(ix)
    ld (_y_element), a

    ;DIMENSIONES
    ld a, pointer_w(ix)
    ld (_w_element), a
    ld a, pointer_hh(ix)
    ld (_h_element), a

    ;Me guardo la direccion para borrarla
    push ix
    pop de

    jr ret_loop