;;----------------------------------------LICENSE NOTICE----------------------------------------
;;  Code & Gfx Copyright (C) 2019 ByteCode Studios (@ByteCodeStudios)
;;  Music (C) 2019 McKlain (@elmcklain)
;;  This file is part of Jackstrad: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;----------------------------------------------------------------------------------------------

.include "pointer_array.h.s"
.include "entity.h.s"
.include "cpctelera.h.s"

;;Numero maximo de entidades con ia
max_ia == 15

;;Numero de entidades actuales
_num_ia_entities:: .db 0

;;Array de entidades
DefinePointerArray _ia_array, max_ia

;;Siguiente elemento a escribir
_last_element_ia:: .dw _ia_array

_next_element_ia:: .dw _ia_array

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Obtiene el numero de entidades
;;REGISTROS DESTRUIDOS: a
;;SALIDA:
;;      a -> Numero entidades
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
man_ia_getNum::
    ld a, (_num_ia_entities)
    ret

;;Obtiene el array con las entidades con IA
;;ENTRADA: hl -> puntero a entidad con ia
man_ia_getArray::
    ld ix, #_ia_array
    ret

;;Obtiene la entidad en ix donde apunta el puntero
;;ENTRADA: ix -> puntero a entidad ia
;;SALIDA: ix -> entidad ia
man_ia_getEntity::
    ld  l, pointer_l(ix)
    ld  h, pointer_h(ix)
    
    ex de,hl
    ld__ixh_d
    ld__ixl_e

    ret

;;Inicializa el array de ia
man_ia_init::
    ;Numero de entidades a 0
    xor a
    ld (_num_ia_entities), a

    ld hl, #_ia_array
    ld (_next_element_ia), hl
    ret

;;;;;;;;;;;;;;;;;;;;;;
;;Limpia todo el array
;;;;;;;;;;;;;;;;;;;;;;
man_ia_clean::
    ld de, #_ia_array
    ld a, #0
    ld bc, #max_ia
    call cpct_memset_asm
    ret


;;Añade una nueva entidad al array de ia
;;ENTRADA: hl -> puntero a entidad con ia
;;SALIDA: hl -> puntero a entidad con ia
man_ia_add::
    push hl

    ;;Guardo la entidad en memoria
    ld  ix, (_next_element_ia)
    ld  pointer_l(ix), l
    ld  pointer_h(ix), h 

    ;Guardamos en la entidad el puntero
    ex de, hl
    ld__ixh_d
    ld__ixl_e
    ld hl, (_next_element_ia)
    ld e_ia_l(ix), l
    ld e_ia_h(ix), h

    ;;Incremento el numero de entidades
    ld hl, #_num_ia_entities
    inc (hl)
  
    ;Calcula la siguiente posicion
    ld hl,  (_next_element_ia)
    ld (_last_element_ia), hl
    ld bc,  #sizeof_pointer
    add hl, bc
    ld (_next_element_ia), hl

    pop hl
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;DESTRUYE DE MEMORIA UNA ENTIDAD
;;REGISTROS DESTRUIDOS: hl,de,bc
;;ENTRADA:
;;      hl -> Puntero que contiene la entidad a borrar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_ia_remove::
    push hl 
    ex de, hl
    
    ;Ponemos a 0 el contenido del ultimo elemento
    ld a, #0
    ld bc, #sizeof_pointer
    call cpct_memset_asm
    pop hl
    ret
