;;----------------------------------------LICENSE NOTICE----------------------------------------
;;  Code & Gfx Copyright (C) 2019 ByteCode Studios (@ByteCodeStudios)
;;  Music (C) 2019 McKlain (@elmcklain)
;;  This file is part of Jackstrad: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;----------------------------------------------------------------------------------------------

.include "pointer_array.h.s"
.include "entity.h.s"
.include "cpctelera.h.s"

;;Numero maximo de entidades con fisicas
max_physics == 50

;;Numero de entidades actuales
_num_physics_entities:: .db 0

;;Array de entidades
DefinePointerArray _physics_array, max_physics

;;Siguiente elemento a escribir
_last_element_physics:: .dw _physics_array

;;Siguiente elemento a escribir
_next_element_physics:: .dw _physics_array 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Obtiene el numero de entidades
;;REGISTROS DESTRUIDOS: a
;;SALIDA:
;;      a -> Numero entidades
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
man_physics_getNum::
    ld a, (_num_physics_entities)
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Obtiene el  array de physicsizado
;;ENTRADA: hl -> puntero a entidad con fisica
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_physics_getArray::
    ld ix, #_physics_array
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Obtiene la entidad en ix donde apunta el puntero
;;ENTRADA: ix -> puntero a entidad con fisica
;;SALIDA: ix -> entidad con fisica
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_physics_getEntity::
    ld  l, pointer_l(ix)
    ld  h, pointer_h(ix)
    
    ex de,hl
    ld__ixh_d
    ld__ixl_e

    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Obtiene la entidad en ix donde apunta el puntero
;;ENTRADA: iy -> puntero a entidad con fisica
;;SALIDA: iy -> entidad con fisica
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_physics_getEntityIY::
    push ix

    push iy
    pop ix
    ld  l, pointer_l(ix)
    ld  h, pointer_h(ix)
    
    ex de,hl
    ld__ixh_d
    ld__ixl_e
    push ix
    pop iy

    pop ix
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Inicializa el array de fisicas
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_physics_init::
    ;Numero de entidades a 0
    xor a
    ld (_num_physics_entities), a

    ld hl, #_physics_array
    ld (_next_element_physics), hl
    call man_physics_clean
    ret

;;;;;;;;;;;;;;;;;;;;;;
;;Limpia todo el array
;;;;;;;;;;;;;;;;;;;;;;
man_physics_clean::
    ld de, #_physics_array
    ld a, #0
    ld bc, #max_physics
    call cpct_memset_asm
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Añade una nueva entidad al array de fisicas
;;ENTRADA: hl -> puntero a entidad fisicas
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_physics_add::
    push hl
    
    ;;Guardo la entidad en memoria
    ld  ix, (_next_element_physics)
    ld  pointer_l(ix), l
    ld  pointer_h(ix), h 

    ;Guardamos en la entidad el puntero
    ex de, hl
    ld__ixh_d
    ld__ixl_e
    ld hl, (_next_element_physics)
    ld e_physics_l(ix), l
    ld e_physics_h(ix), h

    ;;Incremento el numero de entidades
    ld hl, #_num_physics_entities
    inc (hl)
  
    ;Calcula la siguiente posicion
    ld hl,  (_next_element_physics)
    ld (_last_element_physics), hl
    ld bc,  #sizeof_pointer
    add hl, bc
    ld (_next_element_physics), hl
    
    pop hl
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;DESTRUYE DE MEMORIA UNA ENTIDAD
;;REGISTROS DESTRUIDOS: hl,de,bc
;;ENTRADA:
;;      hl -> Puntero que contiene la entidad a borrar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_physics_remove::
    push hl
    
    ;Intercambiamos los valores
    ex de, hl
    ld hl, (_last_element_physics)
    ld bc, #sizeof_pointer
    ldir

    ;Ponemos a 0 el contenido del ultimo elemento
    ld de, (_last_element_physics)
    ld a, #0
    ld bc, #sizeof_pointer
    call cpct_memset_asm

    ;El siguiente elemento apunta al ultimo
    ld hl, (_last_element_physics)
    ld (_next_element_physics), hl

    ;El ultimo elemento se actualiza
    ld bc,#sizeof_pointer
    sbc hl,bc
    ld (_last_element_physics), hl

    ;Decrementamos el numero de entidades
    ld hl, #_num_physics_entities
    dec (hl)

    call man_physics_getLast
    ld (_last_element_physics),hl
    pop hl
    ret

man_physics_getLast::
  ld hl, #_physics_array
  ld a, (_num_physics_entities)

_last_physics_loop:
    dec a
    ret z

    ld bc,#sizeof_pointer
    add hl,bc
   
    jr nz,_last_physics_loop
    ret
