;;----------------------------------------LICENSE NOTICE----------------------------------------
;;  Code & Gfx Copyright (C) 2019 ByteCode Studios (@ByteCodeStudios)
;;  Music (C) 2019 McKlain (@elmcklain)
;;  This file is part of Jackstrad: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;----------------------------------------------------------------------------------------------

.include "entity.h.s"
.include "cpctelera.h.s"
.include "pointer_array.h.s"

;Contiene auxiliares para guardar los punteros
physics_pointer_aux:: .dw #0
ia_pointer_aux:: .dw #0
render_pointer_aux::  .dw #0

;Entrada -> ix(Entidad a borrar del render)
manPointer_removeRender::
    ;Borramos el render
	ld l, e_render_l(ix)
	ld h, e_render_h(ix)
	push hl
	call man_render_remove
    pop hl
    ret

;Entrada -> ix(Entidad a borrar de las fisicas)
manPointer_removePhysics::
    ;Borramos las fisicas
	ld l, e_physics_l(ix)
	ld h, e_physics_h(ix)
	push hl
	call man_physics_remove
    pop hl
    ret

;Entrada -> ix(Entidad a borrar de las fisicas)
manPointer_removeIA::
    ;Borramos las fisicas
	ld l, e_ia_l(ix)
	ld h, e_ia_h(ix)
	push hl
	call man_ia_remove
    pop hl
    ret

;;ENTRADA
;;ix -> Entidad
;;hl -> Puntero a fisica
manPointer_updatePhysics::
    call manPointer_updatePhysicsEntity
	;Meto en bc la entidad
	push ix
	pop bc
	call manPointer_updatePointerPhysics
    ret

;;ENTRADA
;;ix -> Entidad
;;hl -> Puntero a render
manPointer_updateRender::
    call manPointer_updateRenderEntity
	;Meto en bc la entidad
	push ix
	pop bc
	call manPointer_updatePointerRender
    ret

;;ENTRADA
;;ix -> Entidad
;;hl -> Puntero a render
manPointer_updateIA::
    call manPointer_updateIAEntity
	;Meto en bc la entidad
	push ix
	pop bc
	call manPointer_updatePointerIA
    ret

;Entrada -> hl(Valor a actualizar)
;          ix(Entidad a actualizar)
manPointer_updatePhysicsEntity::
    ld e_physics_l(ix), l
	ld e_physics_h(ix), h
    ret

;Entrada -> de(Valor a actualizar)
;          ix(Entidad a actualizar)
manPointer_updateRenderEntity::
    ld e_render_l(ix), l
	ld e_render_h(ix), h
    ret

;Entrada -> bc(Valor a actualizar)
;          ix(Entidad a actualizar)
manPointer_updateIAEntity::
    ld e_ia_l(ix), l
	ld e_ia_h(ix), h
    ret

;Actualiza el lugar donde apunta el render
manPointer_updatePointer::
    ld__ixh_d
    ld__ixl_e
	ld pointer_l(ix),c
	ld pointer_h(ix),b
    ret

;Actualiza el puntero de fisicas
manPointer_updatePointerPhysics::
	ld d, e_physics_h(ix)
	ld e, e_physics_l(ix)
	push ix
    call manPointer_updatePointer
	pop ix
    ret

;Actualiza el puntero de render
manPointer_updatePointerRender::
	ld d, e_render_h(ix)
	ld e, e_render_l(ix)
	push ix
	call manPointer_updatePointer
	pop ix
    ret

;Actualiza el puntero de render
manPointer_updatePointerIA::
	ld d, e_ia_h(ix)
	ld e, e_ia_l(ix)
	push ix
	call manPointer_updatePointer
	pop ix
    ret

;;COMPRUEBA SI UNA ENTIDAD TIENE FISICA
;;ENTRADA: ix -> Entidad
;;SALIDA: a = 1(No tiene fisica)
;;        a = 0(Tiene fisica)
manPointer_checkPhysics::
    

    ret