;;----------------------------------------LICENSE NOTICE----------------------------------------
;;  Code & Gfx Copyright (C) 2019 ByteCode Studios (@ByteCodeStudios)
;;  Music (C) 2019 McKlain (@elmcklain)
;;  This file is part of Jackstrad: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;----------------------------------------------------------------------------------------------

.include "pointer_array.h.s"
.include "entity.h.s"
.include "cpctelera.h.s"

;;Numero maximo de proyectiles
max_projectile == 50

;;Numero de entidades actuales
_num_projectile_entities:: .db 0

;;Array de entidadess
DefinePointerArray _projectile_array, max_projectile

;;Siguiente elemento a escribir
_next_element_projectile:: .dw _projectile_array 

;;Ultimo element del array
_last_element_projectile:: .dw _projectile_array

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Obtiene el numero de entidades
;;REGISTROS DESTRUIDOS: a
;;SALIDA:
;;      a -> Numero entidades
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
man_projectile_getNum::
    ld a, (_num_projectile_entities)
    ret

;;Obtiene el  array de proyectiles
;;ENTRADA: hl -> puntero a entidad projectileizable
man_projectile_getArray::
    ld ix, #_projectile_array
    ret

;;Obtiene la entidad en ix donde apunta el puntero
;;ENTRADA: ix -> puntero a entidad proyectil
;;SALIDA: ix -> entidad proyectil
man_projectile_getEntity::
    ld  l, pointer_l(ix)
    ld  h, pointer_h(ix)
    
    ex de,hl
    ld__ixh_d
    ld__ixl_e

    ret

;;Inicializa el array de proyectil
man_projectile_init::
    ;Numero de entidades a 0
    xor a
    ld (_num_projectile_entities), a

    ld hl, #_projectile_array
    ld (_next_element_projectile), hl
    call man_projectile_clean
    ret


;;Añade una nueva entidad al array de projectileizado
;;ENTRADA: hl -> puntero a entidad proyectil
man_projectile_add::
    push hl
    
    ;;Guardo la entidad en memoria
    ld  ix, (_next_element_projectile)
    ld  pointer_l(ix), l
    ld  pointer_h(ix), h 

    ;;Incremento el numero de entidades
    ld hl, #_num_projectile_entities
    inc (hl)
  
    ;Calcula la siguiente posicion
    ld hl,  (_next_element_projectile)
    ld (_last_element_projectile), hl
    ld bc,  #sizeof_pointer
    add hl, bc
    ld (_next_element_projectile), hl
    
    pop hl
    ret

;;;;;;;;;;;;;;;;;;;;;;
;;Limpia todo el array
;;;;;;;;;;;;;;;;;;;;;;
man_projectile_clean::
    ld de, #_projectile_array
    ld a, #0
    ld bc, #max_projectile
    call cpct_memset_asm
ret

;;Añade render a todos los proyectiles
;;del array
man_projectile_addRender::
    ;;Obtengo el numero de entidades
    call man_projectile_getNum
    
    ;;Compruebo si hay entidades
    or a
	ret z
    call man_projectile_getArray

_projectileloop:
    ;Obtiene la entidad donde apunta 
    push ix
    push af
    call man_projectile_getEntity  
    
    push ix 
    pop hl
    call man_render_add
    
    pop af
    pop ix
    
    ;Siguiente entidad
    ld bc,#sizeof_pointer
    add ix,bc
    dec a
    jr nz,_projectileloop
    
    call man_projectile_init
ret
