;;----------------------------------------LICENSE NOTICE----------------------------------------
;;  Code & Gfx Copyright (C) 2019 ByteCode Studios (@ByteCodeStudios)
;;  Music (C) 2019 McKlain (@elmcklain)
;;  This file is part of Jackstrad: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;----------------------------------------------------------------------------------------------

.include "pointer_array.h.s"
.include "entity.h.s"
.include "cpctelera.h.s"

;;Numero maximo de entidades a renderizar
max_render == 50

;;Numero de entidades actuales
_num_render_entities:: .db 0

;;Array de entidadess
DefinePointerArray _render_array, max_render

;;Siguiente elemento a escribir
_next_element_render:: .dw _render_array 

;;Ultimo element del array
_last_element_render:: .dw _render_array

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Obtiene el numero de entidades
;;REGISTROS DESTRUIDOS: a
;;SALIDA:
;;      a -> Numero entidades
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
man_render_getNum::
    ld a, (_num_render_entities)
    ret

;;Obtiene el  array de renderizado
;;ENTRADA: hl -> puntero a entidad renderizable
man_render_getArray::
    ld ix, #_render_array
    ret

;;Obtiene la entidad en ix donde apunta el puntero
;;ENTRADA: ix -> puntero a entidad renderizable
;;SALIDA: ix -> entidad renderizable
man_render_getEntity::
    ld  l, pointer_l(ix)
    ld  h, pointer_h(ix)
    
    ex de,hl
    ld__ixh_d
    ld__ixl_e

    ret

;;Inicializa el array de render
man_render_init::
    ;Numero de entidades a 0
    xor a
    ld (_num_render_entities), a

    ld hl, #_render_array
    ld (_next_element_render), hl
    call man_render_clean
    ret


;;Añade una nueva entidad al array de renderizado
;;ENTRADA: hl -> puntero a entidad renderizable
man_render_add::
    push hl
    
    ;;Guardo la entidad en memoria
    ld  ix, (_next_element_render)
    ld  pointer_l(ix), l
    ld  pointer_h(ix), h 

    ;Guardamos en la entidad el puntero
    ex de, hl
    ld__ixh_d
    ld__ixl_e
    ld hl, (_next_element_render)
    ld e_render_l(ix), l
    ld e_render_h(ix), h

    ;;Incremento el numero de entidades
    ld hl, #_num_render_entities
    inc (hl)
  
    ;Calcula la siguiente posicion
    ld hl,  (_next_element_render)
    ld (_last_element_render), hl
    ld bc,  #sizeof_pointer
    add hl, bc
    ld (_next_element_render), hl
    
    pop hl
    ret

;;;;;;;;;;;;;;;;;;;;;;
;;Limpia todo el array
;;;;;;;;;;;;;;;;;;;;;;
man_render_clean::
    ld de, #_render_array
    ld a, #0
    ld bc, #max_render
    call cpct_memset_asm
ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;DESTRUYE DE MEMORIA UNA ENTIDAD
;;REGISTROS DESTRUIDOS: hl,de,bc
;;ENTRADA:
;;      hl -> Puntero que contiene la entidad a borrar
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_render_remove::
    push hl

    ex de, hl
    
    ;Intercambiamos los valores
    ld hl, (_last_element_render)
    ld bc, #sizeof_pointer
    ldir

    ;Ponemos a 0 el contenido del ultimo elemento
    ld de, (_last_element_render)
    ld a, #0
    ld bc, #sizeof_pointer
    call cpct_memset_asm

    ;El siguiente elemento apunta al ultimo
    ld hl, (_last_element_render)
    ld (_next_element_render), hl

    ;Decrementamos el numero de entidades
    ld hl, #_num_render_entities
    dec (hl)

    ;Actualizamos el puntero a la ultima entidad
    call man_render_getLast
    ld (_last_element_render),hl
    pop hl

    ret

;;Obtiene el ultimo elemento del array del render
man_render_getLast::
  ld hl, #_render_array
  ld a, (_num_render_entities)

_last_render_loop:
    dec a
    ret z

    ld bc,#sizeof_pointer
    add hl, bc

    jr nz,_last_render_loop
    ret