;;----------------------------------------LICENSE NOTICE----------------------------------------
;;  Code & Gfx Copyright (C) 2019 ByteCode Studios (@ByteCodeStudios)
;;  Music (C) 2019 McKlain (@elmcklain)
;;  This file is part of Jackstrad: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;----------------------------------------------------------------------------------------------

;;ROOMS OF OUR GAME

decompress_buffer = 0x700
levelmaxsize      = 0x1F4
decompress_end    = decompress_buffer + levelmaxsize - 1
tilemap_ptr       = decompress_buffer

loadRoomMenu::
	ld de, #decompress_end
	ld hl, #_levelMenupack_end
	call cpct_zx7b_decrunch_s_asm

	;Cargamos el mapa
  	ld hl, #tilemap_ptr
	call man_map_loadRoom	
	ret

loadRoomMenu2::
	ld de, #decompress_end
	ld hl, #_levelMenu2pack_end
	call cpct_zx7b_decrunch_s_asm

	;Cargamos el mapa
  	ld hl, #tilemap_ptr
	call man_map_loadRoom	
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;; FLOOR 0 ;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

loadRoom0_0::
	;Comprobamos si el que colisiona es el jugador
	call type_checkPlayer
  	or a
  	ret nz

	ld de, #decompress_end
	ld hl, #_level0_0pack_end
	call cpct_zx7b_decrunch_s_asm

	;Cargamos el mapa
  	ld hl, #tilemap_ptr
	call man_map_loadRoom	
	call time_addInterruption

	;Creamos las paredes del mapa
	call man_map_setPlayer
    call man_map_setWallRightPartioned
	call man_map_setWallLeftPartioned
	call man_map_setWallDownPartioned
	call man_map_setWallUp
	

	;Creamos puerta a la izquierda
	ld hl,#loadRoom0_1
	call man_map_setDoorRight

    ld hl,#loadRoom0_2
    call man_map_setDoorLeft

	ld hl,#loadRoom0_3
	call man_map_setDoorDown

	ld b, #19
	ld c, #92
	ld d, #10
	ld e, #55
	call obstacle_create

	ld b, #51
	ld c, #92
	ld d, #10
	ld e, #55
	call obstacle_create

	ld b, #19
	ld c, #70
	ld d, #41
	ld e, #21
	call obstacle_create

	;Creamos al enemigo
    ld b, #15
    ld c, #40
    ld e, #1
    ld d, #1
	ld hl, #patrol1
    call enemy_create

	;Comprobamos si se pone la llave
	ld a,(_hearth_room0)
	or a
	ret z

	ld b, #37
	ld c, #95
	ld a, #0
	call hearth_create
	ret

loadRoom0_1::
	;Comprobamos si es el jugador
	call type_checkPlayer
    or a
    ret nz
	
	ld de, #decompress_end
	ld hl, #_level0_1pack_end
	call cpct_zx7b_decrunch_s_asm

	;Cargamos el mapa
	ld hl, #tilemap_ptr
	call man_map_loadRoom 
	call man_map_setPlayer

	call time_addInterruption2
	
	;Murallas del mapa
	call man_map_setWallLeftPartioned
	call man_map_setWallDownPartioned
	call man_map_setWallRight
	call man_map_setWallUp
	
	;Creamos puerta a la izquierda
	ld hl,#loadRoom0_0
	call man_map_setDoorLeft

	;Creamos puerta abajo
	ld hl,#loadRoom0_5
	call man_map_setDoorDown

	;Cargamos el obstaculo
	 ld b, #51
	 ld c, #141
	 ld d, #18
	 ld e, #35
	 call obstacle_create

	;Creamos al enemigo
    ld b, #15
    ld c, #40
    ld e, #1
    ld d, #1
    ld hl, #patrol02
    call enemy_create

	;Creamos al enemigo
    ld b, #62
    ld c, #50
    ld e, #5
    ld d, #4
    call enemy_create

	;Comprobamos si se pone la llave
	ld a,(_key_room1)
	or a
	ret z

	ld a, #1
	ld b, #64
	ld c, #105
	call key_create

	ret

loadRoom0_2::
    ;Comprobamos si es el jugador
	call type_checkPlayer
    or a
    ret nz
	
    ;Cargamos el mapa
	ld de, #decompress_end
	ld hl, #_level0_2pack_end
	call cpct_zx7b_decrunch_s_asm

  	;Cargamos el mapa
	ld hl, #tilemap_ptr
	call man_map_loadRoom 
	call man_map_setPlayer
	
	;Interrupciones
	call time_addInterruption

    ld hl,#loadRoom0_0
	call man_map_setDoorRight

	ld hl,#loadRoom0_4
	call man_map_setDoorDown

	;Creamos las murallas
	call man_map_setWallRightPartioned
	call man_map_setWallDownPartioned
	call man_map_setWallLeft
	call man_map_setWallUp

	ld b, #10
	ld c, #130
	call plant_create

	;Cargamos el obstaculo
	ld b, #23
	ld c, #61
	ld d, #45
	ld e, #23
	call obstacle_create

	ld b, #55
	ld c, #40
	ld e, #4
	ld d, #2
	call enemy_create

	;Comprobamos si se pone la llave
	ld a,(_key_room2)
	or a
	ret z

	ld a, #2
	ld b, #65
	ld c, #42
	call key_create
	ret

loadRoom0_3::
	;Comprobamos si es el jugador
	call type_checkPlayer
    or a
    ret nz
	
    ;Cargamos el mapa
	ld de, #decompress_end
	ld hl, #_level0_3pack_end
	call cpct_zx7b_decrunch_s_asm

  	;Cargamos el mapa
  	ld hl, #tilemap_ptr
  	call man_map_loadRoom 
	call man_map_setPlayer

	;Interrupciones del mapa
	call time_addInterruption2
	call time_addInterruption2
	call time_addInterruption2

    ld hl,#loadRoom0_0
	call man_map_setDoorUp

	ld hl,#loadRoom0_4
	call man_map_setDoorLeft

	ld hl,#loadRoom0_5
	call man_map_setDoorRight
	

	;Murallas del mapa
	call man_map_setWallLeftPartioned
	call man_map_setWallRightPartioned
	call man_map_setWallDown
	call man_map_setWallUpPartioned

	ret

loadRoom0_4::
	;Comprobamos si es el jugador
	call type_checkPlayer
    or a
    ret nz
	
    ;Cargamos el mapa
 	ld de, #decompress_end
  	ld hl, #_level0_4pack_end
  	call cpct_zx7b_decrunch_s_asm

  	;Cargamos el mapa
  	ld hl, #tilemap_ptr
  	call man_map_loadRoom
	call man_map_setPlayer

	call time_addInterruption2

	call man_map_setWallRightPartioned
	call man_map_setWallUpPartioned
	call man_map_setWallLeft
	call man_map_setWallDown

    ld hl,#loadRoom0_3
	call man_map_setDoorRight

	ld hl,#loadRoom0_2
	call man_map_setDoorUp

	;Cargamos el obstaculo
	ld b, #19
	ld c, #54
	ld d, #42
	ld e, #107
	call obstacle_create

	;Creamos al enemigo
	ld b, #10
    ld c, #158
    ld e, #4
    ld d, #4
    call enemy_create

	;Comprobamos si se pone la llave
	ld a,(_key_room4)
	or a
	ret z

	ld a, #4
	ld b, #11
	ld c, #90
	call key_create
  ret

loadRoom0_5::
	;Comprobamos si es el jugador
	call type_checkPlayer
    or a
    ret nz
	
    ;Cargamos el mapa
  	ld de, #decompress_end
  	ld hl, #_level0_5pack_end
  	call cpct_zx7b_decrunch_s_asm

  	;Cargamos el mapa
  	ld hl, #tilemap_ptr
  	call man_map_loadRoom
	call man_map_setPlayer
	call time_addInterruption2

	;Muros
	call man_map_setWallLeftPartioned
	call man_map_setWallUpPartioned
	call man_map_setWallRight
	call man_map_setWallDown

	;Cargamos el obstaculo
	ld b, #51
	ld c, #40
	ld d, #10
	ld e, #98
	call obstacle_create

	;Puertas
	ld hl,#loadRoom0_3
	call man_map_setDoorLeft

	;Puertas
	ld hl,#loadRoom0_1
	call man_map_setDoorUp

	ld b, #60
    ld c, #150
    ld e, #4
    ld d, #1
    call enemy_create

	ld b, #23
    ld c, #62
    ld e, #4
    ld d, #2
    call enemy_create

	;Comprobamos si se pone la llave
	ld a,(_key_room5)
	or a
	ret z

	ld a, #5
	ld b, #63
	ld c, #40
	call key_create
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;; FLOOR 1 ;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

loadRoom1_0::
	;Comprobamos si el que colisiona es el jugador
	call type_checkPlayer
 	or a
 	ret nz

 	ld de, #decompress_end
 	ld hl, #_level1_0pack_end
 	call cpct_zx7b_decrunch_s_asm

	;Cargamos el mapa
 	ld hl, #tilemap_ptr
	call man_map_loadRoom	
	
	call time_addInterruption2
	call time_addInterruption
	;Interrupciones del mapa
	call time_addInterruption2


	;Creamos las paredes del mapa
	call man_map_setPlayer
    call man_map_setWallRightPartioned
	call man_map_setWallLeftPartioned
	call man_map_setWallDown
	call man_map_setWallUpPartioned
	

	;Creamos puerta a la izquierda
	ld hl,#loadRoom1_1
	call man_map_setDoorRight

    ld hl,#loadRoom1_2
    call man_map_setDoorLeft

	ld hl,#loadRoom1_5
	call man_map_setDoorUp

	;Comprobamos si se pone el corazon
	ld a,(_hearth_room0)
	or a
	ret z

	ld b, #36
	ld c, #95
	ld a, #0
	call hearth_create

	ret

loadRoom1_1::
	;Comprobamos si es el jugador
	call type_checkPlayer
    or a
    ret nz
	
  	ld de, #decompress_end
  	ld hl, #_level1_1pack_end
  	call cpct_zx7b_decrunch_s_asm

  	;Cargamos el mapa
  	ld hl, #tilemap_ptr
  	call man_map_loadRoom 
	call man_map_setPlayer

	call time_addInterruption2
	call time_addInterruption2

	;Murallas del mapa
	call man_map_setWallLeftPartioned
	call man_map_setWallDown
	call man_map_setWallRight
	call man_map_setWallUpPartioned
	
	;Creamos puerta a la izquierda
	ld hl,#loadRoom1_0
	call man_map_setDoorLeft

	;Creamos puerta abajo
	ld hl,#loadRoom1_3
	call man_map_setDoorUp

	;Cargamos el obstaculo
	ld b, #51
	ld c, #40
	ld d, #10
	ld e, #74
	call obstacle_create

	;Creamos al enemigo
    ld b, #62
    ld c, #114
    ld e, #1
    ld d,#1
    ld hl, #patrol12
    call enemy_create

	;Comprobamos si se pone la llave
	ld a,(_key_room1)
	or a
	ret z

	ld a, #1
	ld b, #64
	ld c, #40
	call key_create

	ret

loadRoom1_2::
    ;Comprobamos si es el jugador
	call type_checkPlayer
    or a
    ret nz
	
    ;Cargamos el mapa
  	ld de, #decompress_end
  	ld hl, #_level1_2pack_end
  	call cpct_zx7b_decrunch_s_asm

  	;Cargamos el mapa
  	ld hl, #tilemap_ptr
  	call man_map_loadRoom 
	call man_map_setPlayer
	call time_addInterruption2
	call time_addInterruption2
	call time_addInterruption

    ld hl,#loadRoom1_0
	call man_map_setDoorRight

	ld hl,#loadRoom1_4
	call man_map_setDoorUp

	;Creamos las murallas
	call man_map_setWallRightPartioned
	call man_map_setWallDown
	call man_map_setWallLeft
	call man_map_setWallUpPartioned
	
	;Cargamos el obstaculo
	ld b, #12
	ld c, #132
	ld d, #21
	ld e, #39
	call obstacle_create

	;Cargamos el obstaculo
	ld b, #47
	ld c, #132
	ld d, #21
	ld e, #39
	call obstacle_create

	;Creamos al enemigo
    ld b, #10
    ld c, #40
    ld e, #1
    ld d,#1
    ld hl, #patrol11
    call enemy_create

    ;Comprobamos si se pone la llave
	ld a,(_hearth_room2)
	or a
	ret z

	ld a, #1
	ld b, #37
	ld c, #150
	call hearth_create

	ret

loadRoom1_3::
	;Comprobamos si es el jugador
	call type_checkPlayer
    or a
    ret nz
	
    ;Cargamos el mapa
  	ld de, #decompress_end
  	ld hl, #_level1_3pack_end
  	call cpct_zx7b_decrunch_s_asm

  	;Cargamos el mapa
  	ld hl, #tilemap_ptr
  	call man_map_loadRoom 
	call man_map_setPlayer
	call time_addInterruption
	ld hl,#loadRoom1_5
	call man_map_setDoorLeft

    ld hl,#loadRoom1_1
	call man_map_setDoorDown
	
	;Murallas del mapa
	call man_map_setWallLeftPartioned
	call man_map_setWallRight
	call man_map_setWallDownPartioned
	call man_map_setWallUp

	;Cargamos el obstaculo
	ld b, #19
	ld c, #55
	ld d, #42
	ld e, #108
	call obstacle_create

	;Cargamos el enemigo
	ld b, #10
  	ld c, #37
  	ld a, #1
  	ld d, #1
  	ld hl, #patrol13
  	call enemy_create

  	;Cargamos el enemigo
	ld b, #62
  	ld c, #163
  	ld a, #1
  	ld d, #1
  	ld hl, #patrol14
  	call enemy_create

	;Comprobamos si se pone la llave
	ld a,(_key_room3)
	or a
	ret z

	ld a, #3
	ld b, #65
	ld c, #40
	call key_create

	ret

loadRoom1_4::
	;Comprobamos si es el jugador
	call type_checkPlayer
    or a
    ret nz
	
    ;Cargamos el mapa
 	ld de, #decompress_end
  	ld hl, #_level1_4pack_end
  	call cpct_zx7b_decrunch_s_asm

  	;Cargamos el mapa
  	ld hl, #tilemap_ptr
  	call man_map_loadRoom
	call man_map_setPlayer
	call time_addInterruption2
	call man_map_setWallRightPartioned
	call man_map_setWallUp
	call man_map_setWallLeft
	call man_map_setWallDownPartioned

    ld hl,#loadRoom1_5
	call man_map_setDoorRight

	ld hl,#loadRoom1_2
	call man_map_setDoorDown

	;Cargamos el obstaculo
	ld b, #51
	ld c, #63
	ld d, #16
	ld e, #20
	call obstacle_create

	;Cargamos el enemigo
	ld b, #55
  	ld c, #42
  	ld a, #1
  	ld d, #1
  	ld hl, #patrol15
  	call enemy_create

	;Cargamos el enemigo
	ld b, #12
  	ld c, #61
  	ld a, #1
  	ld d, #1
  	ld hl, #patrol16
  	call enemy_create

	;Comprobamos si se pone la llave
	ld a,(_key_room4)
	or a
	ret z

	ld a, #4
  	ld b, #63
	ld c, #40
	call key_create

	;Comprobamos si se pone el corazon
	ld a,(_hearth_room4)
	or a
	ret z

	ld b, #38
	ld c, #101
	ld a, #4
	call hearth_create

	ret

loadRoom1_5::
	;Comprobamos si es el jugador
	call type_checkPlayer
    or a
    ret nz
	
    ;Cargamos el mapa
  	ld de, #decompress_end
  	ld hl, #_level1_5pack_end
  	call cpct_zx7b_decrunch_s_asm

  	;Cargamos el mapa
  	ld hl, #tilemap_ptr
  	call man_map_loadRoom
	call man_map_setPlayer

	;Muros
	call man_map_setWallLeftPartioned
	call man_map_setWallUp
	call man_map_setWallRightPartioned
	call man_map_setWallDownPartioned

	;Puertas
	ld hl,#loadRoom1_3
	call man_map_setDoorRight

	;Puertas
	ld hl,#loadRoom1_4
	call man_map_setDoorLeft

	;Puertas
	ld hl,#loadRoom1_0
	call man_map_setDoorDown

	;Cargamos el obstaculo
	ld b, #12
	ld c, #62
	ld d, #24
	ld e, #20
	call obstacle_create

	;Cargamos el obstaculo
	ld b, #44
	ld c, #133
	ld d, #24
	ld e, #20
	call obstacle_create

	;Cargamos el enemigo
	ld b, #22
  	ld c, #42
  	ld a, #1
  	ld d, #1
  	ld hl, #patrol17
  	call enemy_create

	;Cargamos el enemigo
	ld b, #62
  	ld c, #157
  	ld a, #1
  	ld d, #1
  	ld hl, #patrol18
  	call enemy_create

	;Comprobamos si se pone la llave
	ld a,(_key_room5)
	or a
	ret z

	ld a, #5
  	ld b, #15
	ld c, #40
	call key_create

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;; FLOOR 2 ;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

loadRoom2_0::
	call type_checkPlayer
    	or a
    	ret nz
	
    ;Cargamos el mapa
	ld de, #decompress_end
	ld hl, #_level2_0pack_end
	call cpct_zx7b_decrunch_s_asm

  	;Cargamos el mapa
	ld hl, #tilemap_ptr
	call man_map_loadRoom 
	call man_map_setPlayer
		call time_addInterruption2
	
    	ld hl, #loadRoom2_1
	call man_map_setDoorRight

	ld hl, #loadRoom2_3
	call man_map_setDoorDown

	;Creamos las murallas
	call man_map_setWallRightPartioned
	call man_map_setWallDownPartioned
	call man_map_setWallLeft
	call man_map_setWallUp

	ld b, #60
	ld c, #150
	ld e, #1
	ld d, #1
	ld hl, #patrol23
	call enemy_create

	ld b, #10
	ld c, #40
	ld e, #1
	ld d, #1
	ld hl, #patrol22
	call enemy_create

	ld a,(_hearth_room0)
	or a
	ret z

	ld b, #37
	ld c, #60
	ld a, #0
	call hearth_create

	ret


loadRoom2_1::
	call type_checkPlayer
    or a
    ret nz
	
    ;Cargamos el mapa
	ld de, #decompress_end
	ld hl, #_level2_1pack_end
	call cpct_zx7b_decrunch_s_asm

  	;Cargamos el mapa
	ld hl, #tilemap_ptr
	call man_map_loadRoom 
			call time_addInterruption2
   	call man_map_setPlayer
    call man_map_setWallRightPartioned
	call man_map_setWallLeftPartioned
	call man_map_setWallDownPartioned
	call man_map_setWallUp
	

	;Creamos puerta a la izquierda
	ld hl,#loadRoom2_2
	call man_map_setDoorRight

	ld hl,#loadRoom2_0
	call man_map_setDoorLeft

	ld hl,#loadRoom2_4
	call man_map_setDoorDown


	ld b, #50
	ld c, #125
	ld e, #4
	ld d, #3
	call enemy_create

	ld b, #20
	ld c, #80
	ld e, #4
	ld d, #1
	call enemy_create

	ld a,(_key_room1)
	or a
	ret z

	ld a, #1
	ld b, #40
	ld c, #40
	call key_create
	ret

loadRoom2_2::
;Comprobamos si es el jugador
	call type_checkPlayer
    or a
    ret nz
	
  	ld de, #decompress_end
  	ld hl, #_level0_1pack_end
  	call cpct_zx7b_decrunch_s_asm

  	;Cargamos el mapa
  	ld hl, #tilemap_ptr
  	call man_map_loadRoom 
	call man_map_setPlayer
			call time_addInterruption2
	ld hl,#loadRoom2_1
	call man_map_setDoorLeft

    ld hl,#loadRoom2_5
	call man_map_setDoorDown
	
	;Murallas del mapa
	call man_map_setWallLeftPartioned
	call man_map_setWallRight
	call man_map_setWallDownPartioned
	call man_map_setWallUp

	ld b, #51
	ld c, #141
	ld d, #18
	ld e, #35
	call obstacle_create

	ld b, #60
	ld c, #120
	ld e, #5
	ld d, #1
	call enemy_create


	;Cargamos el enemigo
	ld b, #10
  	ld c, #37
  	ld e, #1
  	ld d, #1
  	ld hl, #patrol25
  	call enemy_create

  	
	;Comprobamos si se pone la llave
	ld a,(_key_room2)
	or a
	ret z

	ld a, #2
	ld b, #65
	ld c, #40
	call key_create

	ret

loadRoom2_3::
	call type_checkPlayer
    or a
    ret nz
	
    ;Cargamos el mapa
	ld de, #decompress_end
  	ld hl, #_level0_4pack_end
  	call cpct_zx7b_decrunch_s_asm

  	;Cargamos el mapa
  	ld hl, #tilemap_ptr
  	call man_map_loadRoom 
	
	call man_map_setPlayer
			call time_addInterruption2
    ld hl, #loadRoom2_0
	call man_map_setDoorUp

	ld hl, #loadRoom2_4
	call man_map_setDoorRight

	;Creamos las murallas
	call man_map_setWallRightPartioned
	call man_map_setWallUpPartioned
	call man_map_setWallLeft
	call man_map_setWallDown

	ld b, #20
	ld c, #55
	ld d, #40
	ld e, #106
	call obstacle_create

	ld b, #35
	ld c, #165
	ld e, #4
	ld d,#3
	call enemy_create

	ld b, #62
	ld c, #40
	ld e, #4
	ld d,#4
	call enemy_create

	ld a,(_key_room3)
	or a
	ret z

	ld a, #3
	ld b, #10
	ld c, #90
	call key_create
	ret

loadRoom2_4::
	call type_checkPlayer
    or a
    ret nz
	
    ;Cargamos el mapa
	ld de, #decompress_end
	ld hl, #_level0_3pack_end
	call cpct_zx7b_decrunch_s_asm

  	;Cargamos el mapa
	ld hl, #tilemap_ptr
	call man_map_loadRoom 
	call man_map_setPlayer
			call time_addInterruption2
    ld hl, #loadRoom2_1
	call man_map_setDoorUp

	ld hl, #loadRoom2_5
	call man_map_setDoorRight

	ld hl, #loadRoom2_3
	call man_map_setDoorLeft

	;Creamos las murallas
	call man_map_setWallRightPartioned
	call man_map_setWallUpPartioned
	call man_map_setWallLeftPartioned
	call man_map_setWallDown

	ld b, #62
	ld c, #155
	ld e, #5
	ld d, #2
	ld hl, #patrol1
	call enemy_create

	ld b, #10
	ld c, #35
	ld e, #5
	ld d, #4
	ld hl, #patrol1
	call enemy_create
	ret

loadRoom2_5::
	call type_checkPlayer
    or a
    ret nz
	
    ;Cargamos el mapa
	ld de, #decompress_end
	ld hl, #_level2_5pack_end
	call cpct_zx7b_decrunch_s_asm

  	;Cargamos el mapa
	ld hl, #tilemap_ptr
	call man_map_loadRoom 
	call man_map_setPlayer
	call time_addInterruption2
    ld hl, #loadRoom2_2
	call man_map_setDoorUp

	ld hl, #loadRoom2_4
	call man_map_setDoorLeft

	;Creamos las murallas
	call man_map_setWallLeftPartioned
	call man_map_setWallUpPartioned
	call man_map_setWallRight
	call man_map_setWallDown

	ld b, #23
	ld c, #133
	ld d, #34
	ld e, #42
	call obstacle_create

	ld b, #62
	ld c, #45
	ld e, #1
	ld d, #4
	ld hl, #patrol24
	call enemy_create

	ld b, #12
	ld c, #155
	ld e, #5
	ld d,#2
	call enemy_create

	ld a,(_key_room5)
	or a
	jr z, hearth

	ld a, #5
	ld b, #62
	ld c, #155
	call key_create

hearth:
	ld a,(_hearth_room5)
	or a
	ret z

	ld b, #10
	ld c, #45
	ld a, #5
	call hearth_create

	ret
