;;----------------------------------------LICENSE NOTICE----------------------------------------
;;  Code & Gfx Copyright (C) 2019 ByteCode Studios (@ByteCodeStudios)
;;  Music (C) 2019 McKlain (@elmcklain)
;;  This file is part of Jackstrad: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;----------------------------------------------------------------------------------------------

.include "man/man_memory/entity.h.s"
.include "cpctelera.h.s"

DefineEntity player,40,75,8,16,_pj_sp_0,0,0, e_ai_st_noAI,2,6,player_collision, 0, _pj_sp_4
num_dead:: .db 0

counterCollision:: .db 0
;;CREA UN NUEVO ENEMIGO EN LA POSICION INDICADA
;;ENTRADA -> bc(x,y)
;;ENTRADA -> a(Vida)
player_create::
    push bc
	push af
    ld hl, #player
	call entityMan_create
	pop af
    pop bc

	call player_setPosition
	ld e_life(ix), a

	push ix
	call man_render_add
	call man_physics_add
	pop ix
	ret

;;PONE AL JUGADOR EN UNA POSICION
;;ENTRADA -> IX(JUGADOR)
;;		     BC(X,Y)
player_setPosition::
	 ;Posicion del jugador
    ld e_x(ix), b
    ld e_y(ix), c
	ld e_prevX(ix), b
	ld e_prevY(ix), c
	ld e_prevPrevX(ix), b
	ld e_prevPrevY(ix), c
	ret

;;ELIMINA AL JUGADOR
;;ENTRADA -> IX(Enemigo a borrar)
player_remove::
	push ix

    ;Borramos el render
    call manPointer_removeRender
	ld (render_pointer_aux), hl

	;Borramos las fisicas
	call manPointer_removePhysics
	ld (physics_pointer_aux), hl

	;Borramos la entidad
	call entityMan_remove
	pop ix

    ;Comprobamos si es null
	ld a, e_w(ix)
	or a

	;jr c, endGame
	ret z

	;Actualizo las fisicas en la entidad
	ld hl, (physics_pointer_aux)
	or h
	jr z, no_physics_update
	call manPointer_updatePhysics

;No tiene fisica
no_physics_update:
	ld hl, (render_pointer_aux)
	or h
	jr z, no_render_update
	call manPointer_updateRender
;No tiene render
no_render_update:
	ld h, e_ia_h(ix)
	or h
	ret z
	ld l, e_ia_l(ix)
	call manPointer_updateIA
	ret


;;FUNCION QUE MANEJA COMO DEBE ACTUAR 
;;EL PLAYER CUANDO COLISIONA
player_collision::
	;Comprobamos si es un obstaculo
	ld h, a
	
	push bc
	call type_checkObstacle	
	pop bc
	
	;Salta si es un obstaculo
	or a
	jr z, apply_obstacle

	ld a, h
	ld h, a
	push bc
	call type_checkEnemy
	pop bc
	
	;Salta si es un enemigo
	or a
	jr z, _life_less

	ld a, h
	;Comprobamos si es proyectil
    	call type_checkProjectile
    	or a
    	jr z,_life_less_projectile
    	ret

_life_less:
	ld a, (counterCollision)
	or a
	jr nz, _no_life_less
	ld a, #13
	ld (counterCollision), a
	ld a, #10
	ld e_draw(ix), a

_life_less_projectile:
	;Decrementamos la vida del personaje
	ld a, e_life(ix)
	dec a
	ld e_life(ix), a
	
	push ix
	push af
    call interface_draw
	pop af
	pop ix

	or a
	jr nz, no_dead
	
	;Borramos el sprite
	call render_erase_sprite
	call render_switchBuffer
	call render_erase_current_sprite
	call render_switchBuffer
    call player_remove

	ld a, (num_dead)
	inc a
	ld (num_dead), a

	call gameOver
	ld a, #870

wait_press:
	push af
	call cpct_waitVSYNC_asm
	call cpct_waitVSYNC_asm
	call cpct_waitVSYNC_asm
	halt
	pop af

	dec a
	jr nz, wait_press
	
	;Reiniciamos el juego
	ld a, #0
	ld (_floor), a

	ld a, #0
	ld (_keys),a

	call game_clearData 
	call man_map_loadNextFloor
	ret
    
	
_no_life_less:
    
	dec a
    ld (counterCollision), a
no_dead:
	
	ld a, (counterCollision)
	or a
	ret nz
	ld a, #6
	ld e_draw(ix), a
	ret

;;bc->xy
;;de->wh
;;ix->personaje
apply_obstacle:
	call collision_detect_size

	cp #1
	jr z, _neg_y_down
	;Left
	cp #2
	jr z, _neg_x_left
	;Right
	cp #3
	jr z, _neg_x_right
	cp #4
	jr z, _neg_y_up

	ret

_neg_x_left:
	;Comprobamos si la vx es positiva
	ld a, e_vx(ix)
	bit 7, a
	ret z

	ld a, #1
	ld (neg_x), a
	ld a, e_vx(ix)
    neg
    ld e_vx(ix),a
	ret 

_neg_x_right:
	;Comprobamos si la vx es negativa
	ld a, e_vx(ix)
	bit 7, a
	ret nz

	ld a, #1
	ld (neg_x), a

	ld a, e_vx(ix)
    neg
    ld e_vx(ix),a
	ret

_neg_y_down:
	;Comprobamos si la vy es positiva
	ld a, e_vy(ix)
	bit 7, a
	ret nz

	ld a, #1
	ld (neg_y), a

	ld a, e_vy(ix)
    neg
    ld e_vy(ix),a
	ret

_neg_y_up:
	;Comprobamos si la vy es negativa
	ld a, e_vy(ix)
	bit 7, a
	ret z

	ld a, #1
	ld (neg_y), a

	ld a, e_vy(ix)
    neg
    ld e_vy(ix),a
	ret

;;Aumenta la vida del personaje
player_incLife::
	push ix
	
	call entityMan_getArray
	ld a, e_life(ix)
	cp #6
	jr z,end_incLife

	inc a
	ld e_life(ix), a

end_incLife:
	pop ix
	ret


endGame:
  	;call render_switchBuffer
  	;call render_back
  	;call render_switchBuffer
  	;call render_back

	