;;----------------------------------------LICENSE NOTICE----------------------------------------
;;  Code & Gfx Copyright (C) 2019 ByteCode Studios (@ByteCodeStudios)
;;  Music (C) 2019 McKlain (@elmcklain)
;;  This file is part of Jackstrad: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;----------------------------------------------------------------------------------------------

;;
;;Projectile Manager
;;

.include "man/man_memory/entity.h.s"
.include "man/man_memory/pointer_array.h.s"
.include "cpctelera.h.s"

DefineEntity projectile,0,0,4,8,_bala_sp_0,0,0, e_ai_st_noAI, 1,1, man_projectile_collision, 2,_bala_sp_1

;Valor de colocacion del proyectil a la izquierda
value_left_shot = 5
value_up_shot = 8
value_down_shot = 8
value_right_shot = 5

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Creates a projectile
;;Salida: ix-> pointer to the new projectile
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_projectile_create::
	ld hl, #projectile
	
	call entityMan_create
	push ix
	call man_render_add
	call man_physics_add
	pop ix
	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,,
;;Set direction of the projectile
;;
;;Input: b->vx, c->vy
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,
man_projectile_set_direction::
	ld e_vx(ix), b
	ld e_vy(ix), c
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Set the relative position
;;
;;Input: ix->projectile entity, b->x, c->y, d->offwidth, e->offheight
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
man_projectile_set_position::
	ld a, e_vy(ix)
	or a
	jr z, _shoot_x 
	bit 7, a
	jr z, _shoot_y_down
	
	;Disparo arriba
	ld a,c
	ld h, #value_up_shot
	sub h
	ld e_y(ix), a

	jr _y

_shoot_y_down:

	ld a, c
	add e
	add e
	ld e,#value_down_shot
	add e
	ld e_y(ix), a
;Calcula la mitad de la x
_y:
	ld a, b
	add a, d
	ld e_x(ix), a
	jp _end

;Comprueba la direccion del disparo x
_shoot_x:
	ld a, e_vx(ix)
	bit 7, a
	jr nz, _shoot_x_left
	ld a, b
	add a,d
	add a,d
	ld d, #value_right_shot
	add d
	ld e_x(ix), a
	
	jp _x

_shoot_x_left:
	ld a,b
	ld h, #value_left_shot
	sub h
	ld e_x(ix), a
_x:
	ld a, c
	add a, e
	ld e_y(ix), a

_end:
	ret

;;Borra de memoria un proyectil
;;IX -> Proyectil a borrar
man_projectile_remove::
	push ix

    ;Borramos el render
    call manPointer_removeRender
	ld (render_pointer_aux), hl

	;Borramos las fisicas
	call manPointer_removePhysics
	ld (physics_pointer_aux), hl

	;Borramos la entidad
	call entityMan_remove
	pop ix

    ;Comprobamos si es null
	ld a, e_w(ix)
	or a
	ret z

	;Actualizo las fisicas en la entidad
	ld hl, (physics_pointer_aux)
	or h
	jr z, no_physics_update
	call manPointer_updatePhysics

;No tiene fisica
no_physics_update:
	ld hl, (render_pointer_aux)
	or h
	jr z, no_render_update
	call manPointer_updateRender

;No tiene render
no_render_update:
	ld h, e_ia_h(ix)
	or h
	ret z
	ld l, e_ia_l(ix)
	call manPointer_updateIA
	ret

man_projectile_collision::
	ld a,e_prevPrevX(ix)
    or a
    jr z, no_erase

	call render_erase_sprite
	call render_switchBuffer
	call render_erase_current_sprite
	call render_switchBuffer
no_erase:
	call man_projectile_remove
	ret


	
