;;----------------------------------------LICENSE NOTICE----------------------------------------
;;  Code & Gfx Copyright (C) 2019 ByteCode Studios (@ByteCodeStudios)
;;  Music (C) 2019 McKlain (@elmcklain)
;;  This file is part of Jackstrad: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;----------------------------------------------------------------------------------------------

.include "man/man_memory/entity.h.s"
.include "cpctelera.h.s"
.include "man/patrol.h.s"
.include "man/man_memory/pointer_array.h.s"

_counterShoot:: .db 0
_counterShootMove:: .db 0
_counterEnt:: .db 0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Inicia el sistema de control
;;Entrada: IX=puntero entidades
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_ai_control_init::
	call man_ia_getArray		
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;AI STAND BY
;;
;;Input: ix->pointer array
;;;Destroy: hl,a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_ai_stand_by::

	ld a, e_ai_prev_st(ix)
	cp #5
	jr z, _no_patrol

	cp #4
	jr z, _multiple

	;call man_patrol_get
	;ld e_ai_patrol_step_l(ix), l
	;;ld e_ai_patrol_step_h(ix), h
	ld e_ai_prev_st(ix), #e_ai_st_stand_by
	ld e_ai_st(ix), #e_ai_st_patrol
	
	;;Charge the next function
	ld hl, #sys_ai_patrol
	ld e_ai_function_l(ix), l
	ld e_ai_function_h(ix),h
	ret

_no_patrol:
	ld e_ai_prev_st(ix), #e_ai_st_stand_by
	ld e_ai_st(ix), #e_ai_st_attack_direction
	ld hl, #sys_ai_attack_direction
	ld e_ai_function_l(ix), l
	ld e_ai_function_h(ix),h
	ret
_multiple::
	ld e_ai_prev_st(ix), #e_ai_st_stand_by
	ld e_ai_st(ix), #e_ai_st_attack_multiple
	ld hl, #sys_ai_attack_multiple
	ld e_ai_function_l(ix), l
	ld e_ai_function_h(ix),h
	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Attack in the four directions, one direction for iteration
;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_ai_attack_multiple::
	;ld a, (_counterShoot)
	;or a
	;jr z, _end_multiple

	call sys_ai_attack_direction

	ld a, e_direction(ix)
	inc a
	cp #5
	jr nz, _end_multiple
	ld a, #1

_end_multiple:
	ld e_direction(ix), a
	ld e_ai_prev_st(ix), #e_ai_st_attack_multiple
	ld e_ai_st(ix), #e_ai_st_stand_by
	ld hl, #sys_ai_attack_multiple
	ld e_ai_function_l(ix), l
	ld e_ai_function_h(ix),h
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Attack in a direction
;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_ai_attack_direction::

	ld a, (_counterShoot)
    or a
    jr nz, _dec_counter

	push ix
	call man_projectile_create
	push ix
	pop hl
	pop ix
	ld a, e_direction(ix)
	push hl
	cp #1
	jr z, _shoot_left2
	;Left
	cp #2
	jr z, _shoot_down2
	;Right
	cp #3
	jr z, _shoot_right2
	cp #4
	jr z, _shoot_up2
	
	
_shoot_left2:
	call enemy_setLeftAnimation
	ld b, #-2
	ld c, #0
	jp _end2
_shoot_right2:
	call enemy_setRightAnimation
	ld b, #2
	ld c, #0
	jp _end2

_shoot_down2:
	call enemy_setDownAnimation
	ld b, #0
	ld c, #-3
	jp _end2
_shoot_up2:
	call enemy_setUpAnimation
	ld b, #0
	ld c, #3
	

_end2:
	pop hl
	ex de, hl
	ld h, d
	ld l, e

	push ix						;;save entity ia
	
	ld__ixh_d
    	ld__ixl_e

    	call man_projectile_set_direction

    	pop ix                                       ;;restore entity ia


    	ld b, e_x(ix)
    	ld c, e_y(ix)
    	ld d, e_offsetW(ix)
    	ld e, e_offsetH(ix)

    	push ix							;;save entity ia
    	push de							;;save offwidth and offheight

    	;;restore entity projectile
    	ex de, hl
    	ld__ixh_d
    	ld__ixl_e

    	pop de							;;restore offwidth and offheight

    	call man_projectile_set_position

    	pop ix
    	
    	ld a, #14
    	ld (_counterShoot), a
    	jp _endAttack	
    									;;restore entity ia
_dec_counter:
   	dec a
    	ld (_counterShoot), a

_endAttack:
	ld hl, #sys_ai_stand_by
	ld a, e_ai_st(ix)
	ld e_ai_prev_st(ix), a
	ld e_ai_st(ix), #e_ai_st_stand_by
	ld e_ai_function_l(ix), l
	ld e_ai_function_h(ix),h
	
    	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;AI MOVE TO
;;
;;Input: ix -> pointer entity
;;Destroy: hl,a
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_ai_move_to::

	ld a, (_counterShootMove)
    	or a
    	jr nz, _no_shoot

_shoot:
	ld a, #1
	ld (_counterShootMove), a
	call sys_ai_attack

_no_shoot:
	dec a
	ld (_counterShootMove), a
	ld a, e_ai_aim_x(ix)	;; A = X_objetivo
	sub e_x(ix)			;; A = X_objetivo - X

	jr nc, _move_right_or_stop   ;;Comprobamos si el resultado es positivo o 0

_move_left:
	ld e_vx(ix),#-1

	jr _end_x

_move_right_or_stop:
	jr z, _arrive_x
	ld e_vx(ix),#1
	jr _end_x

_arrive_x:
	ld e_vx(ix),#0
	jr _y

_end_x:
	ret

_y:
	;; A = Y_objetivo
	ld a, e_ai_aim_y(ix)
	;; A = Y_objetivo - X	
	sub e_y(ix)			
	;;Comprobamos si el resultado es positivo o 0
	jr nc, _move_down_or_stop   
_move_up:
	ld e_vy(ix),#-2
	ret

_move_down_or_stop:
	jr z, _arrive_y
	ld e_vy(ix),#2
	ret
_arrive_y:

	ld e_vy(ix),#0
	ld a, e_vx(ix)
	or a
	jr nz, _end_y
	;;ld a, e_ai_prev_st(ix)
	ld e_ai_st(ix), #e_ai_st_patrol
	ld e_ai_prev_st(ix), #e_ai_st_move_to
	;;charge the next function
	ld hl, #sys_ai_patrol
	ld e_ai_function_l(ix), l
	ld e_ai_function_h(ix), h

_end_y:
	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,,
;;AI PATROL
;;Input: IX -> pointer to the entity
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_ai_patrol::

	ld l, e_ai_patrol_step_l(ix)       
	ld h, e_ai_patrol_step_h(ix)		

	;;Set x,y in the ai
	ld a, (hl)    			;;a = x
	cp #patrol_invalid_move_x
	jr z, _init_patrol

	ld e_ai_aim_x(ix), a
	inc hl
	ld a, (hl)     			;;a = y
	ld e_ai_aim_y(ix), a

	;;set the next position
	inc hl
	ld e_ai_patrol_step_l(ix), l       
	ld e_ai_patrol_step_h(ix), h

	ld e_ai_prev_st(ix), #e_ai_st_patrol
	ld e_ai_st(ix), #e_ai_st_move_to
	;;charge the next function
	ld hl, #sys_ai_move_to
	ld e_ai_function_l(ix), l
	ld e_ai_function_h(ix),h
	ret

_init_patrol:				;;reset the patrol

	inc hl
	ld a, (hl)
	inc hl
	ld h, (hl)

	ld e_ai_patrol_step_l(ix), a
	ld e_ai_patrol_step_h(ix), h

	ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Ataca
;;Entrada: IX=puntero entidades, A=número de entidades
;;Destruye: AF,BC,DE,IX
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_ai_attack::
	
	push ix
	call man_projectile_create
	push ix
	pop hl

	call entityMan_getArray

	ld b, e_x(ix)				;;x player
	ld c, e_y(ix)				;;y player

	pop ix					;;restore entity ai

	ld a, e_y(ix)			;;y1
	sub c					;;y2

	ld d, a				;;y3

	ld a, e_x(ix)                 ;;x1
	sub b					;;x2 and x3 = a

	
	ld c,a
	call AbsA				;;absolute value of the x1-x2
	push af
	ld a, d
	ld e, d
	call AbsA				;;absolute value of the y1-y2
	ld d, a
	pop af
	
	cp d
	jr c, _shoot_y

_shoot_x:	
	bit 7, c
	jr nz, _shoot_right
	
_shoot_left:
	push af 
	push hl
	call enemy_setLeftAnimation
	call enemy_setCounterIA
	pop hl
	pop af
	ld b, #-2
	ld c, #0
	jp _end
_shoot_right:
	push af 
	push hl
	call enemy_setRightAnimation
	call enemy_setCounterIA
	pop hl
	pop af
	ld b, #2
	ld c, #0
	jp _end

_shoot_y:

	bit 7, e
	jr nz, _shoot_down
_shoot_up:
	push af 
	push hl
	call enemy_setDownAnimation
	call enemy_setCounterIA
	pop hl
	pop af
	ld b, #0
	ld c, #-3
	jp _end
_shoot_down:
	push af 
	push hl
	call enemy_setUpAnimation
	call enemy_setCounterIA
	pop hl
	pop af
	ld b, #0
	ld c, #3

_end:

	;;restore entity projectile
	ex de, hl
	ld h, d
	ld l, e

	push ix						;;save entity ia
	
	ld__ixh_d
    ld__ixl_e

    	call man_projectile_set_direction

    	pop ix                                       ;;restore entity ia


    	ld b, e_x(ix)
    	ld c, e_y(ix)
    	ld d, e_offsetW(ix)
    	ld e, e_offsetH(ix)

    	push ix							;;save entity ia
    	push de							;;save offwidth and offheight

    	;;restore entity projectile
    	ex de, hl
    	ld__ixh_d
    	ld__ixl_e

    	pop de							;;restore offwidth and offheight

    	call man_projectile_set_position

    	pop ix						;;restore entity ia

	ret

AbsA:
 bit 7,a
 ret z
 neg
 ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Actualiza todas las entidades IA
;;Entrada: IX=puntero entidades, A=número de entidades
;;Destruye: AF,BC,DE,IX
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_ai_control_update::
	call man_ia_getNum
	or a
	ret z
	call man_ia_getArray	
_loop:

_AI_ent:
	push af
	push ix

	ld a, 0(ix)
	or a
	jr z, here

	call man_ia_getEntity
	call enemy_setAnimation
	ld h, e_ai_function_h(ix)
	ld l, e_ai_function_l(ix)
	ld bc, #here
	push bc
	jp(hl)
here:
	pop ix
	pop af
	dec a					
	ret z				;;comprobamos si ya no quedan más entidades

	ld de, #sizeof_pointer
	add ix, de

	jr _loop
