;;----------------------------------------LICENSE NOTICE----------------------------------------
;;  Code & Gfx Copyright (C) 2019 ByteCode Studios (@ByteCodeStudios)
;;  Music (C) 2019 McKlain (@elmcklain)
;;  This file is part of Jackstrad: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;----------------------------------------------------------------------------------------------

;;SISTEMA DE COLISIONES

.include "cpctelera.h.s"
.include "man/man_memory/entity.h.s"
.include "man/man_memory/pointer_array.h.s"
_x:: .db 0
_y:: .db 0
collisions_init::
	ret

collision_detect_size::
	ld a, b
	ld (_x), a
	ld a, c
	ld (_y), a

	ld a, e_y(ix)
	ld h, #1
	sub h
	ld l, a
	ld a, e_x(ix)
	ld h, #1
	add h
	ld h,a

	ld a, e_w(ix)
	ld b, #3
	sub b
	ld b,a
	ld c, #1
	call collision_check_collision_part
	or a
	jr z, _up

	;;creation of a bounding box at left of the player
	ld a, e_x(ix)
	ld h, #1
	sub h
	ld h,a			;;x bounding box
	ld a, e_y(ix)
	ld l, #1
	add l
	ld l,a		;;y bounding box

	ld b, #1			;;weight bounding box
	ld a, e_h(ix)
	ld c, #3
	sub c
	ld c,a		;;heigh bounding box



	call collision_check_collision_part
	or a
	jr z, _left

	;;we prepare the data for the right bounding box
	ld a, e_x(ix)
	ld h, e_w(ix)
	add h
	ld h, a
	ld a, e_y(ix)
	ld l, #1
	add l
	ld l,a		;;y bounding box

	ld b, #1
	ld a, e_h(ix)
	ld c, #3
	sub c
	ld c,a		;;heigh bounding box

	call collision_check_collision_part
	or a
	jr z, _right
	;;we prepare the data for the up bounding box
	ld a, e_y(ix)
	ld h, e_h(ix)
	add h
	ld l, a
	ld a, e_x(ix)
	ld h, #1
	add h
	ld h,a

	ld a, e_w(ix)
	ld b, #3
	sub b
	ld b,a
	ld c, #1

	call collision_check_collision_part
	or a
	jr z, _down

	ld a, #0
	jp _end

_down:
	;;down
	ld a, #1
	jp _end

_left:
	ld a, #2
	jp _end
_right:

	ld a, #3
	jp _end
_up:
	ld a, #4
_end:	
	ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;,
;;Look if there is a collision
;;bc->wh otro entidad
;;de->wh entidad
;;hl->xy otro entidad
;;(_x)->x entidad
;;(_y)->y entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
collision_check_collision_part::
	push de
	ld a, (_x)
	ld e, a

	ld a, h
	add b
	sub e
	pop de
	ret c

	ld a, (_x)
	add d
	sub h
	ret c

	ld a, (_y)
	ld b, a

	;Comprueba eje y
	ld a,l
	add c
	sub b
	ret c

	ld a, b
	add e
	sub l
	ret c
	;COLISIONA
	ld a,#0
	ret
;_no_collision_part:
;	ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; COMPRUEBA SI HAY COLISIÓN ENTRE DOS CUERPOS
;;
;;ENTRADA:
;;      ix -> Puntero que contiene la entidad
;;      iy -> Puntero con la otra entidad
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
collisions_check::
	;Comprueba eje x
	call man_physics_getEntity
	call man_physics_getEntityIY
	
	ld a, e_x(ix)
	add e_w(ix)
	sub e_x(iy)
	jr c, _no_collision
	
	ld a, e_x(iy)
	add e_w(iy)
	sub e_x(ix)
	jr c, _no_collision

	;Comprueba eje y
	ld a,e_y(ix)
	add e_h(ix)
	sub e_y(iy)
	jr c, _no_collision

	ld a, e_y(iy)
	add e_h(iy)
	sub e_y(ix)
	jr c, _no_collision

	;COLISIONA
	;Meto a la funcion colision de la primera entidad
	push ix
	pop hl
	;Le paso de que tipo es la entidad
	ld a, e_type(iy)
	ld b, e_x(iy)
    ld c, e_y(iy)
    ld d, e_w(iy)
    ld e, e_h(iy) 
	push ix
	call man_col_add
	pop ix

	;Meto a la funcion colision de la segunda entidad
	push iy
	pop hl
	;Le paso de que tipo es la entidad
	ld a, e_type(ix)
	ld b, e_x(ix)
    ld c, e_y(ix)
    ld d, e_w(ix)
    ld e, e_h(ix) 
	call man_col_add

	ret
_no_collision:
	ret

;Loop que comprueba una entidad con todas las demas
check_collision_loop::
   ;Cargamos la siguiente entidad a iy
   push ix
   pop iy
   ld bc, #sizeof_pointer
   add iy, bc
_collision_loop:
   	;Me guardo los valores antes de comprobar
	;si colisionan
    push af
	push ix
	push iy
    call collisions_check
	pop iy
	pop ix
    pop af

    dec a
	ret z
	;Aumento 'iy' para pasar a la siguiente 
	;entidad
    ld bc,#sizeof_pointer
    add iy,bc
    jr nz,_collision_loop
    
    ret

;;FUNCION QUE COMPRUEBA LAS COLISIONES
;;DE TODAS LAS ENTIDADES
collision_update::
   	;Obtengo la primera posicion del array
  	call man_physics_getNum
    or a
	ret z
    
    call man_physics_getArray

  	dec a
   	ret z

update_loop:
   ;Guardamos los registros antes de la llamada
   push af
   call check_collision_loop
   pop af

   dec a
   ;Siguiente entidad
   ld bc, #sizeof_pointer
   add ix, bc
   jr nz,update_loop
   ret


;;1) Comprobar si el objeto colisiona en los ejes x e y con otro objeto.


;	- - - - - - - - - -  -> y0+height 					- - - - - - - -> y1+height
;	'			'							'		'
;	'			'							'		'
;	'			'							'		'
;	'			'							- - - - - - - -> y1
;	- - - - - - - - - -  -> y1 						|		|
;	|			|							x1	 x1+width
;	x0		x0+width

;; if((x0 <= x1 && x0+width >= x1) || (x0 >= x1 && x0 <= x1+width)) && ((y0 <= y1 && y0+height >= y1) || (y0 >= y1 && y0 <= y1+height))
;;		COLISIÓN
;; else
;;		NO COLISION



;;2) En caso de colisionar, impedir movimientos.


