;;----------------------------------------LICENSE NOTICE----------------------------------------
;;  Code & Gfx Copyright (C) 2019 ByteCode Studios (@ByteCodeStudios)
;;  Music (C) 2019 McKlain (@elmcklain)
;;  This file is part of Jackstrad: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;----------------------------------------------------------------------------------------------

.include "cpctelera.h.s"
.include "man/man_memory/entity.h.s"

;;ld__ixh_d
;;ld__ixl_e Carga en la parte baja de ix e
_counter_wait = 10

_counter:: .db 0
_useJoystick:: .db 0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;RECIBE EL INPUT DEL TECLADO Y APLICA EL MOVIMIENTO
;;CORRESPONDIENTE
;;REGISTROS DESTRUIDOS: hl
;;ENTRADA:
;;      ix -> Puntero que contiene al jugador
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_control_update::   
    call entityMan_getArray  
    
    ld a, e_type(ix)
    ;Comprobamos si es el jugador
    call type_checkPlayer
    or a
    ret nz   

    ;Comprobamos si usamos el joystick 
    ld a, #1
    ld (_useJoystick), a
    call control_move

    ld a,(_useJoystick)
    or a
    jr nz, no_keyboard
    call control_move_keyboard

no_keyboard:
    ld a, (_counter)
    or a
    jr nz, dec_counter

    call control_attack  
    ret

dec_counter:
    dec a
    ld (_counter), a
    ret

control_move_keyboard::
    ;;El movimiento vertical está invertido, arriba:negativo, abajo:positivo
   ;; call cpct_scanKeyboard_asm          
    ld e_vx(ix), #0                         ;;ponemos velocidad x a 0
    ld e_vy(ix), #0                         ;;ponemos velocidad y a 0

    ld hl, #Key_O
    call cpct_isKeyPressed_asm
    jr z, o_no_pulsada
    ld e_vx(ix), #-1                        ;;si se pulsa A se aumenta la velocidad hacia la izquierda
     
    call player_setLeftAnimation
    ret
o_no_pulsada:
    ld hl, #Key_P
    call cpct_isKeyPressed_asm
    jr z, p_no_pulsada
    ld e_vx(ix), #1                      ;;si pulsamos D aumenta a la derecha
    
    call player_setRightAnimation
    ret
p_no_pulsada:
    ld e_vx(ix), #0

    ld hl,  #Key_Q
    call cpct_isKeyPressed_asm
    jr z, q_no_pulsada
    ld e_vy(ix), #-2                      ;;si pulsamos W aumenta hacia arriba

    call player_setUpAnimation
    ret
q_no_pulsada:
    ld hl, #Key_A
    call cpct_isKeyPressed_asm
    jr z, a_no_pulsada
    ld e_vy(ix), #2                      ;;si pulsamos S aumenta hacia abajo
    call player_setDownAnimation
    ret
a_no_pulsada:
    ld e_vy(ix), #0
    ret  


control_move:
    ;;El movimiento vertical está invertido, arriba:negativo, abajo:positivo
    ;; call cpct_scanKeyboard_asm          
    ld e_vx(ix), #0                         ;;ponemos velocidad x a 0
    ld e_vy(ix), #0                         ;;ponemos velocidad y a 0

    ld hl, #Joy0_Left
    call cpct_isKeyPressed_asm
    jr z, left_no_pulsada
    ld e_vx(ix), #-1                        ;;si se pulsa A se aumenta la velocidad hacia la izquierda
     
    call player_setLeftAnimation
    ret
left_no_pulsada:
    ld hl, #Joy0_Right
    call cpct_isKeyPressed_asm
    jr z, right_no_pulsada
    ld e_vx(ix), #1                      ;;si pulsamos D aumenta a la derecha
    
    call player_setRightAnimation
    ret
right_no_pulsada:
    ld e_vx(ix), #0

    ld hl, #Joy0_Up
    call cpct_isKeyPressed_asm
    jr z, up_no_pulsada
    ld e_vy(ix), #-2                      ;;si pulsamos W aumenta hacia arriba

    call player_setUpAnimation
    ret
up_no_pulsada:
    ld hl, #Joy0_Down
    call cpct_isKeyPressed_asm
    jr z, down_no_pulsada
    ld e_vy(ix), #2                      ;;si pulsamos S aumenta hacia abajo
    call player_setDownAnimation
    ret
down_no_pulsada:
    ld e_vy(ix), #0
    
    ld a, #0
    ld (_useJoystick), a
    ret  

control_attack:
    ;;Leemos si esta pulsando 'L'
    ;;call cpct_scanKeyboard_asm  
    ld hl, #Joy0_Fire1
    call cpct_isKeyPressed_asm
    jr nz, no_check_space

    ld hl, #Key_Space
    call cpct_isKeyPressed_asm
    ret z
no_check_space:  
    ld a, (player_orientation)
    cp #2
    jr nz, no_attack_right

    ld a, #_counter_wait
    ld (_counter), a
   
    ;Calculamos la direccion de ataque
    ld b, #2
    ld c, #0
    jp _end

no_attack_right:
     ;;Leemos si esta pulsando 'J'
    ld a, (player_orientation)
    or a
    jr nz, no_attack_left
    
    ld a, #_counter_wait
    ld (_counter), a
    ld b, #-2
    ld c, #0
    jp _end

no_attack_left:
    ;;Leemos si esta pulsando 'I'
    ld a, (player_orientation)
    cp #1
    jr nz, no_attack_up
   
    ld a, #_counter_wait
    ld (_counter), a
    ld b, #0
    ld c, #-4
    jp _end

no_attack_up:
    ;;Leemos si esta pulsando 'I'
    ld a, (player_orientation)
    cp #3
    ret nz

    ld a, #_counter_wait
    ld (_counter), a
   
    ld b, #0
    ld c, #4
    jp _end

k_no_pulsada:
    ret
    
_end::
    ;;put the vx and vy in the stack
    push bc
    call man_projectile_create
    pop bc                                  ;;restore vx and vy in bc

    call man_projectile_set_direction

    push ix                                 ;;put the pointer to the projectile in stack
    call entityMan_getArray
    ld b, e_x(ix)
    ld c, e_y(ix)
    ld d, e_offsetW(ix)
    ld e, e_offsetH(ix)
    pop ix                                  ;;restore the pointer to the projectile in ix

    call man_projectile_set_position
    ret

