;;----------------------------------------LICENSE NOTICE----------------------------------------
;;  Code & Gfx Copyright (C) 2019 ByteCode Studios (@ByteCodeStudios)
;;  Music (C) 2019 McKlain (@elmcklain)
;;  This file is part of Jackstrad: an Amstrad CPC Game made with CPCTelera
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;----------------------------------------------------------------------------------------------

.include "man/man_memory/entity.h.s"
.include "../main.h.s"
.include "man/man_memory/pointer_array.h.s"
.include "cpctelera.h.s"
;;SISTEMA DE RENDER

front_buffer: .db 0xC0
back_buffer:: .db 0x80
_counterDraw:: .db 0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; LIMPIA EL BACK BUFFER
;; Destruye HL, DE, BC
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_initBuffer:
    ld a, (back_buffer)
    ld h, a
    ld de,#0x0
    ld l, e
    ld bc, #0x4000
    call cpct_memset_f64_asm
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; RENDER_INIT
;; Inicia los valores del render
;; Destruye AF, BC, DE, HL
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_init::
    ld c, #0
    call cpct_setVideoMode_asm
    ld hl, #_palette
    ld de, #16
    call cpct_setPalette_asm
    cpctm_setBorder_asm HW_BLACK


    ;Render Buffer
    call render_initBuffer    
    call render_back
    call render_switchBuffer
    call render_back
    ret

render_updatePosition::
     ;Actualizamos la previa de la previa
    ld c, e_prevX(ix)
    ld b, e_prevY(ix)
    ld e_prevPrevX(ix), c
    ld e_prevPrevY(ix), b

    ld c, e_x(ix) ;; X
    ld b, e_y(ix) ;; Y

    ;Actualizamos la previa
    ld e_prevX(ix),c
    ld e_prevY(ix),b 
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;DIBUJA EN PANTALLA UNA ENTIDAD
;;REGISTROS DESTRUIDOS: de,hl,bc,af
;;ENTRADA:
;;      ix -> Puntero que contiene la entidad
;;      de -> Video Memory location
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_draw::
     ld a,e_prevPrevX(ix)
     or a
     jr z, no_erase
     call render_erase_sprite

no_erase:    
    push ix
    
    ld a, e_draw(ix)
    or a
    jr nz, no_dibuja
    ;;Obtenemos la posicion de
    ;;pantalla actual
    ;;DE = Back buffer
    ld a,(back_buffer)
    ld d,a
    ld e, #0
    ld c, e_x(ix) ;; X
    ld b, e_y(ix) ;; Y

    call cpct_getScreenPtr_asm
    ;;Posicion -> hl
     ex  de, hl
    ;Informacion para pintar el sprite
    
    ld l, e_sprite_l(ix)
    ld h, e_sprite_h(ix)
    ld c, e_w(ix)
    ld b, e_h(ix)
    

    call cpct_drawSprite_asm
    pop ix
    ret

no_dibuja:
    dec a
    ld e_draw(ix),a

    pop ix
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Draw the part of a map where a sprite was
;;
;;Input: ix->entidad
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_erase_sprite::
    ;;Cargo el buffer
    ld a,(back_buffer)
    ld d,a
    ld e, #0
    ld c, e_prevPrevX(ix) ;; X
    ld b, e_prevPrevY(ix) ;; Y
    call cpct_getScreenPtr_asm
    ex  de, hl
    
    push ix
    ;;Cargo el sprite
   
    ld b, e_h(ix)
    ld c, e_w(ix)
    ld l, e_spriteDel_l(ix)
    ld h, e_spriteDel_h(ix)
    call cpct_drawSprite_asm
    pop ix
    ret

render_erase_current_sprite::
    ;;Cargo el buffer
    ld a,(back_buffer)
    ld d,a
    ld e, #0
    ld c, e_x(ix) ;; X
    ld b, e_y(ix) ;; Y
    call cpct_getScreenPtr_asm

    ;;Cargo el sprite
    ex de, hl
    ;ld hl, #_recu_sp
    ;ld b, e_h(ix)
    ;ld c, e_w(ix)
      
    ;call cpct_drawSprite_asm
    push ix
    ;;Cargo el sprite
   
    ld b, e_h(ix)
    ld c, e_w(ix)
    ld l, e_spriteDel_l(ix)
    ld h, e_spriteDel_h(ix)
    
    call cpct_drawSprite_asm
    pop ix
    ret
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Dibuja en pantalla todas las entidades
;;REGISTROS DESTRUIDOS: de,hl,bc,af
;;ENTRADA:
;;      ix -> Puntero que contiene la primera posicion
;;            del array de entidades
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_update_entities::
    ;;Obtengo el numero de entidades
    call man_render_getNum
    
    ;;Compruebo si hay entidades
    or a
	ret z
    call man_render_getArray
_renloop:
    ;Guardo el array de punteros
    push ix
    push af

    ;Obtiene la entidad donde apunta 
    call man_render_getEntity  
    ld a,e_prevX(ix)
    or a
    jr z, no_render
    push ix 
    call render_draw
    pop ix
 
no_render: 
    call render_updatePosition
    pop af
    pop ix
    
    ;Decremento el numero de entidades
    dec a
    
    ;Siguiente entidad
    ld bc,#sizeof_pointer
    add ix,bc
    jr nz,_renloop
    
    ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Draw the background
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_back::
    ld bc, #0x1914          ;;h = 0x19 = 25 = 200/8 | w = 0x14 = 20 = 80/4
    ld de, #20
    ld hl, #_pruTile_00
    call cpct_etm_setDrawTilemap4x8_ag_asm

    ld a, (back_buffer)
    ld d, a
    ld e, #0
    
    ex de, hl
    ld de, (_mapa)

    call cpct_etm_drawTilemap4x8_ag_asm

    ret

;;Actualizamos el render en cada iteracion
render_update::
    ;Comprobamos si hay que cambiar el mapa
    ld a, (_change)
    cp #0
    jr z, _no_change_map
    
    call render_back
    
    ;Draw interface
    call drawLife
    call drawLetter
    call drawLetterKey
    call drawKeys
    call render_switchBuffer
    call render_back 
    
    ;Draw interface
    call drawLife
    call drawLetter
    call drawLetterKey
    call drawKeys

    ld a, #0
    ld (_change), a
_no_change_map:
    call render_update_entities
    ret

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;Cambia el buffer donde se dibuja
;;REGISTROS DESTRUIDOS: l
;;ENTRADA:
;;      ix -> Puntero que contiene la primera posicion
;;            del array de entidades
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
render_switchBuffer::
    ld a,(back_buffer)
    ld b,a
    ld a, (front_buffer)
    ld (back_buffer),a
    ld a, b
    ld (front_buffer), a

    srl b
    srl b
    ld l, b
    jp cpct_setVideoMemoryPage_asm
